# Copyright (c) 2016 AT&T Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest
from unittest import mock

from muranodashboard.categories import tables
from muranodashboard.categories import views


class TestCategoriesView(unittest.TestCase):

    def setUp(self):
        super(TestCategoriesView, self).setUp()

        self.categories_view = views.CategoriesView()
        self.categories_view._prev = False
        self.categories_view._more = False

        mock_request = mock.Mock()
        self.categories_view.request = mock_request

        self.assertEqual(tables.CategoriesTable,
                         self.categories_view.table_class)
        self.assertEqual('categories/index.html',
                         self.categories_view.template_name)
        self.assertEqual('Application Categories',
                         self.categories_view.page_title)

        mock_horizon_utils = mock.patch.object(views, 'utils').start()
        mock_horizon_utils.get_page_size.return_value = 2
        self.addCleanup(mock.patch.stopall)

    def test_has_prev_data(self):
        self.assertFalse(self.categories_view.has_prev_data(None))

    def test_has_more_data(self):
        self.assertFalse(self.categories_view.has_more_data(None))

    @mock.patch.object(views, 'api', autospec=True)
    def test_get_data(self, mock_api):
        """Test that get_data works."""
        mock_client = mock_api.muranoclient(mock.Mock())
        mock_client.categories.list.return_value = [
            'foo_cat', 'bar_cat'
        ]
        self.categories_view.request.GET.get.return_value = 'foo_marker'

        result = self.categories_view.get_data()

        expected_categories = ['bar_cat', 'foo_cat']
        expected_kwargs = {
            'filters': {},
            'marker': 'foo_marker',
            'sort_dir': 'asc',
            'limit': 3
        }

        self.assertEqual(expected_categories, result)
        self.assertTrue(self.categories_view.has_more_data(None))
        self.assertFalse(self.categories_view.has_prev_data(None))
        self.categories_view.request.GET.get.assert_called_once_with(
            tables.CategoriesTable._meta.prev_pagination_param, None)
        mock_client.categories.list.assert_called_once_with(
            **expected_kwargs)

    @mock.patch.object(views, 'api', autospec=True)
    def test_get_data_with_more_results(self, mock_api):
        """Test that get_data with pagesize smaller than result size works."""
        mock_client = mock_api.muranoclient(mock.Mock())
        mock_client.categories.list.return_value = [
            'foo_cat', 'bar_cat', 'baz_cat'
        ]
        self.categories_view.request.GET.get.return_value = 'foo_marker'

        result = self.categories_view.get_data()

        # Only two results should have been returned, with categories reversed.
        expected_categories = ['bar_cat', 'foo_cat']
        expected_kwargs = {
            'filters': {},
            'marker': 'foo_marker',
            'sort_dir': 'asc',
            'limit': 3
        }

        self.assertEqual(expected_categories, result)
        self.assertTrue(self.categories_view.has_more_data(None))
        self.assertTrue(self.categories_view.has_prev_data(None))
        self.categories_view.request.GET.get.assert_called_once_with(
            tables.CategoriesTable._meta.prev_pagination_param, None)
        mock_client.categories.list.assert_called_once_with(
            **expected_kwargs)

    @mock.patch.object(views, 'api', autospec=True)
    def test_get_data_with_desc_sort_dir(self, mock_api):
        """Test that get_data with sort_dir = 'desc' works."""
        mock_client = mock_api.muranoclient(mock.Mock())
        mock_client.categories.list.return_value = [
            'foo_cat', 'bar_cat'
        ]
        self.categories_view.request.GET.get.side_effect = [None, 'bar_marker']

        result = self.categories_view.get_data()

        expected_categories = ['foo_cat', 'bar_cat']
        expected_kwargs = {
            'filters': {},
            'marker': 'bar_marker',
            'sort_dir': 'desc',
            'limit': 3
        }

        self.assertEqual(expected_categories, result)
        self.assertFalse(self.categories_view.has_more_data(None))
        self.assertTrue(self.categories_view.has_prev_data(None))
        self.categories_view.request.GET.get.assert_has_calls([
            mock.call(tables.CategoriesTable._meta.prev_pagination_param,
                      None),
            mock.call(tables.CategoriesTable._meta.pagination_param, None)
        ])
        mock_client.categories.list.assert_called_once_with(
            **expected_kwargs)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import urllib.parse as urlparse

from django.core.files import storage
from django import http
from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
# django.contrib.formtools migration to django 1.8
# https://docs.djangoproject.com/en/1.8/ref/contrib/formtools/
try:
    from django.contrib.formtools.wizard import views as wizard_views
except ImportError:
    from formtools.wizard import views as wizard_views
from horizon import exceptions
from horizon.forms import views
from horizon import messages
from horizon import tables as horizon_tables
from horizon.utils import functions as utils
from horizon import views as horizon_views
from muranoclient.common import exceptions as exc
from muranoclient.common import utils as muranoclient_utils
from openstack_dashboard.api import glance
from openstack_dashboard.api import keystone
from oslo_log import log as logging

from muranodashboard import api
from muranodashboard.api import packages as pkg_api
from muranodashboard.catalog import views as catalog_views
from muranodashboard.common import utils as muranodashboard_utils
from muranodashboard.environments import consts
from muranodashboard.packages import consts as packages_consts
from muranodashboard.packages import forms
from muranodashboard.packages import tables

LOG = logging.getLogger(__name__)

FORMS = [('upload', forms.ImportPackageForm),
         ('modify', forms.UpdatePackageForm),
         ('add_category', forms.SelectCategories)]

BUNDLE_FORMS = [('upload', forms.ImportBundleForm), ]


def is_app(wizard):
    """Check if we're uploading an application

    Return true if uploading package is an application.
    In that case, category selection form need to be shown.
    """
    step_data = wizard.storage.get_step_data('upload')
    if step_data:
        return step_data['package'].type == 'Application'
    return False


def _ensure_images(name, package, request, step_data=None):
    glance_client = glance.glanceclient(
        request, version='2')

    base_url = packages_consts.MURANO_REPO_URL
    image_specs = package.images()

    try:
        imgs = muranoclient_utils.ensure_images(
            glance_client=glance_client,
            image_specs=image_specs,
            base_url=base_url)
        for img in imgs:
            msg = _("Trying to add {0} image to glance. "
                    "Image will be ready for deployment after "
                    "successful upload").format(img['name'],)
            messages.warning(request, msg)
            log_msg = _("Trying to add {0}, {1} image to "
                        "glance. Image will be ready for "
                        "deployment after successful upload")\
                .format(img['name'], img['id'],)
            LOG.info(log_msg)
            if step_data:
                step_data['images'].append(img)
    except Exception as e:
        msg = _("Error {0} occurred while installing "
                "images for {1}").format(e, name)
        messages.error(request, msg)
        LOG.exception(msg)


class PackageDefinitionsView(horizon_tables.DataTableView):
    table_class = tables.PackageDefinitionsTable
    template_name = 'packages/index.html'
    page_title = _("Packages")

    _more = False
    _prev = False

    def has_more_data(self, table):
        return self._more

    def has_prev_data(self, table):
        return self._prev

    def get_data(self):
        sort_dir = self.request.GET.get('sort_dir', 'asc')
        opts = {
            'include_disabled': True,
            'sort_dir': sort_dir,
        }
        marker = self.request.GET.get(
            tables.PackageDefinitionsTable._meta.pagination_param, None)

        opts = self.get_filters(opts)

        packages = []
        page_size = utils.get_page_size(self.request)
        with api.handled_exceptions(self.request):
            packages, extra = pkg_api.package_list(
                self.request, marker=marker, filters=opts, paginate=True,
                page_size=page_size)

            if sort_dir == 'asc':
                self._more = extra
            else:
                packages = list(reversed(packages))
                self._prev = extra

            if packages:
                if sort_dir == 'asc':
                    backward_marker = packages[0].id
                    opts['sort_dir'] = 'desc'
                else:
                    backward_marker = packages[-1].id
                    opts['sort_dir'] = 'asc'

                __, extra = pkg_api.package_list(
                    self.request, filters=opts, paginate=True,
                    marker=backward_marker, page_size=0)

                if sort_dir == 'asc':
                    self._prev = extra
                else:
                    self._more = extra

        # Add information about project tenant for admin user
        if self.request.user.is_superuser:
            tenants = []
            try:
                tenants, _more = keystone.tenant_list(self.request)
            except Exception:
                exceptions.handle(self.request,
                                  _("Unable to retrieve project list."))
            tenent_name_by_id = {tenant.id: tenant.name for tenant in tenants}
            for i, p in enumerate(packages):
                packages[i].tenant_name = tenent_name_by_id.get(p.owner_id)
        else:
            current_tenant = self.request.session['token'].tenant
            for i, package in enumerate(packages):
                if package.owner_id == current_tenant['id']:
                    packages[i].tenant_name = current_tenant['name']
                else:
                    packages[i].tenant_name = _('UNKNOWN')
        return packages

    def get_context_data(self, **kwargs):
        context = super(PackageDefinitionsView,
                        self).get_context_data(**kwargs)
        context['tenant_id'] = self.request.session['token'].tenant['id']
        return context

    def get_filters(self, filters):
        filter_action = self.table._meta._filter_action
        if filter_action:
            filter_field = self.table.get_filter_field()
            if filter_action.is_api_filter(filter_field):
                filter_string = self.table.get_filter_string()
                if filter_field and filter_string:
                    filters[filter_field] = filter_string
        return filters


class ImportBundleWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                         wizard_views.SessionWizardView):
    template_name = 'packages/import_bundle.html'
    page_title = _("Import Bundle")

    def get_context_data(self, **kwargs):
        context = super(ImportBundleWizard, self).get_context_data(**kwargs)
        repo_url = urlparse.urlparse(packages_consts.MURANO_REPO_URL)
        context['murano_repo_url'] = "{}://{}".format(
            repo_url.scheme, repo_url.netloc)
        return context

    def get_form_initial(self, step):
        initial_dict = self.initial_dict.get(step, {})
        if step == 'upload':
            for name in ['url', 'name', 'import_type']:
                if name in self.request.GET:
                    initial_dict[name] = self.request.GET[name]
        return initial_dict

    def process_step(self, form):
        @catalog_views.update_latest_apps
        def _update_latest_apps(request, app_id):
            LOG.info('Adding {0} application to the'
                     ' latest apps list'.format(app_id))

        step_data = self.get_form_step_data(form)
        if self.steps.current == 'upload':
            import_type = form.cleaned_data['import_type']
            data = {}
            f = None
            base_url = packages_consts.MURANO_REPO_URL

            if import_type == 'by_url':
                f = form.cleaned_data['url']
            elif import_type == 'by_name':
                f = muranoclient_utils.to_url(
                    form.cleaned_data['name'],
                    path='bundles/',
                    base_url=base_url,
                    extension='.bundle',
                )

            try:
                bundle = muranoclient_utils.Bundle.from_file(f)
            except Exception as e:
                if '(404)' in e.message:
                    msg = _("Bundle creation failed."
                            "Reason: Can't find Bundle name from repository.")
                else:
                    msg = _("Bundle creation failed."
                            "Reason: {0}").format(e)
                LOG.exception(msg)
                messages.error(self.request, msg)
                raise exceptions.Http302(
                    reverse('horizon:app-catalog:packages:index'))

            for package_spec in bundle.package_specs():
                try:
                    package = muranoclient_utils.Package.from_location(
                        package_spec['Name'],
                        version=package_spec.get('Version'),
                        url=package_spec.get('Url'),
                        base_url=base_url,
                        path=None,
                    )
                except Exception as e:
                    msg = _("Error {0} occurred while parsing package {1}")\
                        .format(e, package_spec.get('Name'))
                    messages.error(self.request, msg)
                    LOG.exception(msg)
                    continue

                reqs = package.requirements(base_url=base_url)
                for dep_name, dep_package in reqs.items():
                    _ensure_images(dep_name, dep_package,
                                   self.request)

                    try:
                        files = {dep_name: dep_package.file()}
                        package = api.muranoclient(
                            self.request).packages.create(data, files)
                        messages.success(
                            self.request,
                            _('Package {0} uploaded').format(dep_name)
                        )
                        _update_latest_apps(
                            request=self.request, app_id=package.id)
                    except exc.HTTPConflict:
                        msg = _("Package {0} already registered.").format(
                            dep_name)
                        messages.warning(self.request, msg)
                        LOG.exception(msg)
                    except exc.HTTPException as e:
                        reason = muranodashboard_utils.parse_api_error(
                            getattr(e, 'details', ''))
                        if not reason:
                            raise
                        msg = _("Package {0} upload failed. {1}").format(
                            dep_name, reason)
                        messages.warning(self.request, msg)
                        LOG.exception(msg)
                    except Exception as e:
                        msg = _("Importing package {0} failed. "
                                "Reason: {1}").format(dep_name, e)
                        messages.warning(self.request, msg)
                        LOG.exception(msg)
                        continue

        return step_data

    def done(self, form_list, **kwargs):
        redirect = reverse('horizon:app-catalog:packages:index')
        msg = _('Bundle successfully imported.')
        LOG.info(msg)
        messages.success(self.request, msg)
        return http.HttpResponseRedirect(str(redirect))


class ImportPackageWizard(horizon_views.PageTitleMixin, views.ModalFormMixin,
                          wizard_views.SessionWizardView):
    file_storage = storage.FileSystemStorage(location=consts.CACHE_DIR)
    template_name = 'packages/upload.html'
    condition_dict = {'add_category': is_app}
    page_title = _("Import Package")

    def get_form_initial(self, step):
        initial_dict = self.initial_dict.get(step, {})
        if step == 'upload':
            for name in ['url', 'repo_name', 'repo_version', 'import_type']:
                if name in self.request.GET:
                    initial_dict[name] = self.request.GET[name]
        return initial_dict

    def get_context_data(self, **kwargs):
        context = super(ImportPackageWizard, self).get_context_data(**kwargs)
        repo_url = urlparse.urlparse(packages_consts.MURANO_REPO_URL)
        context['murano_repo_url'] = "{}://{}".format(
            repo_url.scheme, repo_url.netloc)
        return context

    def done(self, form_list, **kwargs):
        data = self.get_all_cleaned_data()
        app_id = self.storage.get_step_data('upload')['package'].id
        # Remove package file from result data
        for key in ('package', 'import_type', 'url',
                    'repo_version', 'repo_name'):
            del data[key]

        dep_pkgs = self.storage.get_step_data('upload').get(
            'dependencies', [])

        installed_images = self.storage.get_step_data('upload').get(
            'images', [])

        redirect = reverse('horizon:app-catalog:packages:index')
        dep_data = {'enabled': data['enabled'],
                    'is_public': data['is_public']}
        murano_client = api.muranoclient(self.request)
        for dep_pkg in dep_pkgs:
            try:
                murano_client.packages.update(dep_pkg.id, dep_data)
                LOG.debug('Success update for package {0}.'.format(dep_pkg.id))
            except Exception as e:
                msg = _("Couldn't update package {0} parameters. Error: {1}")\
                    .format(dep_pkg.fully_qualified_name, e)
                LOG.warning(msg)
                messages.warning(self.request, msg)

        # Images have been imported as private images during the 'upload' step
        # If the package is public, make the required images public
        if data['is_public']:
            try:
                glance_client = glance.glanceclient(self.request, '1')
            except Exception:
                glance_client = None

            if glance_client:
                for img in installed_images:
                    try:
                        glance_client.images.update(img['id'], is_public=True)
                        LOG.debug(
                            'Success update for image {0}'.format(img['id']))
                    except Exception as e:
                        msg = _("Error {0} occurred while setting image {1}, "
                                "{2} public").format(e, img['name'], img['id'])
                        messages.error(self.request, msg)
                        LOG.exception(msg)
            elif len(installed_images):
                msg = _("Couldn't initialise glance v1 client, "
                        "therefore could not make the following images "
                        "public: {0}").format(' '.join(
                            [img['name'] for img in installed_images]))
                messages.warning(self.request, msg)
                LOG.warning(msg)

        try:
            data['tags'] = [t.strip() for t in data['tags'].split(',')]
            murano_client.packages.update(app_id, data)
        except exc.HTTPForbidden:
            msg = _("You are not allowed to change"
                    " this properties of the package")
            LOG.exception(msg)
            exceptions.handle(
                self.request, msg,
                redirect=reverse('horizon:app-catalog:packages:index'))
        except (exc.HTTPException, Exception):
            LOG.exception(_('Modifying package failed'))
            exceptions.handle(self.request,
                              _('Unable to modify package'),
                              redirect=redirect)
        else:
            msg = _('Package parameters successfully updated.')
            LOG.info(msg)
            messages.success(self.request, msg)
            return http.HttpResponseRedirect(str(redirect))

    def _handle_exception(self, original_e):
        reason = ''
        if hasattr(original_e, 'details'):
            try:
                error = json.loads(original_e.details).get('error')
                if error:
                    reason = error.get('message')
            except ValueError:
                raise original_e
        msg = _('Uploading package failed. {0}').format(reason)
        LOG.exception(msg)
        exceptions.handle(
            self.request,
            msg,
            redirect=reverse('horizon:app-catalog:packages:index'))

    def process_step(self, form):
        @catalog_views.update_latest_apps
        def _update_latest_apps(request, app_id):
            LOG.info('Adding {0} application to the'
                     ' latest apps list'.format(app_id))

        step_data = self.get_form_step_data(form).copy()
        if self.steps.current == 'upload':
            import_type = form.cleaned_data['import_type']
            data = {}
            f = None
            base_url = packages_consts.MURANO_REPO_URL

            if import_type == 'upload':
                pkg = form.cleaned_data['package']
                f = pkg.file
            elif import_type == 'by_url':
                f = form.cleaned_data['url']
            elif import_type == 'by_name':
                name = form.cleaned_data['repo_name']
                version = form.cleaned_data['repo_version']
                f = muranoclient_utils.to_url(
                    name, version=version,
                    path='apps/',
                    extension='.zip',
                    base_url=base_url,
                )

            try:
                package = muranoclient_utils.Package.from_file(f)
                name = package.manifest['FullName']
            except Exception as e:
                if '(404)' in e.message:
                    msg = _("Package creation failed."
                            "Reason: Can't find Package name from repository.")
                else:
                    msg = _("Package creation failed."
                            "Reason: {0}").format(e)
                LOG.exception(msg)
                messages.error(self.request, msg)
                raise exceptions.Http302(
                    reverse('horizon:app-catalog:packages:index'))

            reqs = package.requirements(base_url=base_url)
            original_package = reqs.pop(name)
            step_data['dependencies'] = []
            step_data['images'] = []
            for dep_name, dep_package in reqs.items():
                _ensure_images(dep_name, dep_package, self.request, step_data)

                try:
                    files = {dep_name: dep_package.file()}
                    package = api.muranoclient(self.request).packages.create(
                        data, files)
                    messages.success(
                        self.request,
                        _('Package {0} uploaded').format(dep_name)
                    )
                    _update_latest_apps(
                        request=self.request, app_id=package.id)
                    step_data['dependencies'].append(package)
                except exc.HTTPConflict:
                    msg = _("Package {0} already registered.").format(
                        dep_name)
                    messages.warning(self.request, msg)
                    LOG.exception(msg)
                except Exception as e:
                    msg = _("Error {0} occurred while "
                            "installing package {1}").format(e, dep_name)
                    messages.error(self.request, msg)
                    LOG.exception(msg)
                    continue

            # add main packages images
            _ensure_images(name, original_package, self.request, step_data)

            # import main package itself
            try:
                files = {name: original_package.file()}
                package = api.muranoclient(self.request).packages.create(
                    data, files)
                messages.success(self.request,
                                 _('Package {0} uploaded').format(name))
                _update_latest_apps(request=self.request, app_id=package.id)

                step_data['package'] = package

            except exc.HTTPConflict:
                msg = _("Package with specified name already exists")
                LOG.exception(msg)
                exceptions.handle(
                    self.request,
                    msg,
                    redirect=reverse('horizon:app-catalog:packages:index'))
            except exc.HTTPInternalServerError as e:
                self._handle_exception(e)

            except exc.HTTPException as e:
                reason = muranodashboard_utils.parse_api_error(
                    getattr(e, 'details', ''))
                if not reason:
                    raise
                LOG.exception(reason)
                exceptions.handle(
                    self.request,
                    reason,
                    redirect=reverse('horizon:app-catalog:packages:index'))

            except Exception as original_e:
                self._handle_exception(original_e)

        return step_data

    def get_form_kwargs(self, step=None):
        kwargs = {}
        if step == 'add_category':
            kwargs.update({'request': self.request})
        if step == 'modify':
            package = self.storage.get_step_data('upload').get('package')
            kwargs.update({'package': package, 'request': self.request})
        return kwargs


class ModifyPackageView(views.ModalFormView):
    form_class = forms.ModifyPackageForm
    template_name = 'packages/modify_package.html'
    success_url = reverse_lazy('horizon:app-catalog:packages:index')
    failure_url = reverse_lazy('horizon:app-catalog:packages:index')
    page_title = _("Modify Package")

    def get_initial(self):
        app_id = self.kwargs['app_id']
        package = api.muranoclient(self.request).packages.get(app_id)
        return {
            'package': package,
            'app_id': app_id,
        }

    def get_context_data(self, **kwargs):
        context = super(ModifyPackageView, self).get_context_data(**kwargs)
        context['app_id'] = self.kwargs['app_id']
        context['type'] = self.get_form().initial['package'].type
        return context


class DetailView(horizon_views.HorizonTemplateView):
    template_name = 'packages/detail.html'
    page_title = "{{ app.name }}"

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        app = self.get_data()
        context["app"] = app
        return context

    def get_data(self):
        app = None
        try:
            app_id = self.kwargs['app_id']
            app = api.muranoclient(self.request).packages.get(app_id)
        except Exception:
            INDEX_URL = 'horizon:app-catalog:packages:index'
            exceptions.handle(self.request,
                              _('Unable to retrieve package details.'),
                              redirect=reverse(INDEX_URL))
        return app


def download_packge(request, app_name, app_id):
    try:
        body = api.muranoclient(request).packages.download(app_id)

        content_type = 'application/octet-stream'
        response = http.HttpResponse(body, content_type=content_type)
        response['Content-Disposition'] = 'filename={name}.zip'.format(
            name=app_name)

        return response
    except exc.HTTPException:
        LOG.exception(_('Something went wrong during package downloading'))
        redirect = reverse('horizon:app-catalog:packages:index')
        exceptions.handle(request,
                          _('Unable to download package.'),
                          redirect=redirect)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import unittest

from muranodashboard.dynamic_ui import helpers


class TestHelper(unittest.TestCase):
    def test_to_str(self):
        names = ['string', b'ascii', u'ascii',
                 u'\u043d\u0435 \u0430\u0441\u043a\u0438']
        for name in names:
            self.assertIsInstance(helpers.to_str(name), str)

    def test_int2base(self):
        for x in range(30):
            self.assertEqual("{0:b}".format(x), helpers.int2base(x, 2))
            self.assertEqual("{0:o}".format(x), helpers.int2base(x, 8))
            self.assertEqual("{0:x}".format(x), helpers.int2base(x, 16))

    def test_camelize(self):
        snake_name = "snake_case_name"
        camel_name = helpers.camelize(snake_name)
        self.assertEqual("SnakeCaseName", camel_name)

    def test_explode(self):
        not_string = 123456
        explode_int = helpers.explode(not_string)
        self.assertEqual(123456, explode_int)

        string = "test"
        explode_str = helpers.explode(string)
        self.assertEqual(['t', 'e', 's', 't'], explode_str)

    def test_insert_hidden_ids(self):
        app_dict = {'?': {
            'type': 'test.App',
            'id': '123'}
        }
        app_list = [1, 2, 3, 4]

        dict_result = helpers.insert_hidden_ids(app_dict)
        list_result = helpers.insert_hidden_ids(app_list)
        self.assertEqual('test.App', dict_result['?']['type'])
        self.assertEqual(app_list, list_result)



# Copyright (c) 2016 AT&T Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import json
from unittest import mock

from muranodashboard.environments import consts
from muranodashboard.environments import topology

from openstack_dashboard.test import helpers


class TestTopology(helpers.APITestCase):

    def setUp(self):
        super(TestTopology, self).setUp()
        self.mock_request = mock.Mock()

    @mock.patch.object(topology, 'reverse')
    @mock.patch.object(topology, 'pkg_cli')
    def test_get_app_image_with_package(self, mock_pkg_cli, mock_reverse):
        mock_package = mock.Mock(id='test_pkg_id')
        mock_pkg_cli.app_by_fqn.return_value = mock_package
        mock_reverse.return_value = '/foo/bar/baz'

        url = topology.get_app_image(self.mock_request, 'test_app_fqn')
        self.assertEqual('/foo/bar/baz', url)
        mock_reverse.assert_called_once_with(
            "horizon:app-catalog:catalog:images", args=('test_pkg_id',))

    @mock.patch.object(topology, 'pkg_cli')
    def test_get_app_image_without_package(self, mock_pkg_cli):
        mock_pkg_cli.app_by_fqn.return_value = None

        for status in (consts.STATUS_ID_DEPLOY_FAILURE,
                       consts.STATUS_ID_DELETE_FAILURE):
            url = topology.get_app_image(self.mock_request, 'test_app_fqn',
                                         status)
            self.assertEqual('/static/dashboard/img/stack-red.svg', url)

        url = topology.get_app_image(self.mock_request, 'test_app_fqn',
                                     consts.STATUS_ID_READY)
        self.assertEqual('/static/dashboard/img/stack-green.svg', url)

        for status in (consts.STATUS_ID_PENDING, consts.STATUS_ID_DEPLOYING,
                       consts.STATUS_ID_DELETING, consts.STATUS_ID_NEW):
            url = topology.get_app_image(self.mock_request, 'test_app_fqn',
                                         status)
            self.assertEqual('/static/dashboard/img/stack-gray.svg', url)

    def test_get_environment_status_message(self):
        for status, expected_msg in (
                (consts.STATUS_ID_DEPLOYING, 'Deployment is in progress'),
                (consts.STATUS_ID_DEPLOY_FAILURE, 'Deployment failed')):
            mock_entity = mock.Mock(status=status)
            in_progress, status_msg =\
                topology._get_environment_status_message(mock_entity)
            self.assertTrue(in_progress)
            self.assertEqual(expected_msg, status_msg)

    def test_get_environment_status_message_misc_status(self):
        mock_entity = mock.Mock(status='foo_status')
        in_progress, status_msg =\
            topology._get_environment_status_message(mock_entity)
        self.assertTrue(in_progress)
        self.assertEqual('', status_msg)

    def test_get_environment_status_message_in_progress(self):
        for status, expected_msg in (
                (consts.STATUS_ID_PENDING, 'Waiting for deployment'),
                (consts.STATUS_ID_READY, 'Deployed')):
            mock_entity = mock.Mock(status=status)
            in_progress, status_msg =\
                topology._get_environment_status_message(mock_entity)
            self.assertFalse(in_progress)
            self.assertEqual(expected_msg, status_msg)

            fake_entity = {'?': {'status': status}}
            in_progress, status_msg =\
                topology._get_environment_status_message(fake_entity)
            self.assertFalse(in_progress)
            self.assertEqual(expected_msg, status_msg)

    def test_truncate_type(self):
        self.assertEqual('foo', topology._truncate_type('foo', 4))
        self.assertEqual('...bar', topology._truncate_type('foo.bar', 4))
        self.assertEqual('foo.bar', topology._truncate_type('foo.bar', 7))

    @mock.patch.object(topology, 'loader')
    @mock.patch.object(topology, 'pkg_cli')
    def test_render_d3_data(self, mock_pkg_cli, mock_loader):
        mock_pkg_cli.app_by_fqn.return_value = None
        mock_loader.render_to_string.return_value = 'test_env_info'

        fake_services = [
            {
                '?': {
                    'id': 'test_service_id',
                    'status': consts.STATUS_ID_READY,
                    'type': 'io.murano.resources.foo',
                },
                'name': 'foo',
                'instance': {
                    '?': {
                        'id': 'test_instance_id',
                        'type': 'io.murano.resources.bar',
                    },
                    'assignFloatingIp': True,
                    'extra': [{'name': 'bar'}],
                    'name': 'bar',
                    'ipAddresses': ['127.0.0.1']
                }
            },
            {
                '?': {
                    'id': 'test_alt_service_id',
                    'status': consts.STATUS_ID_PENDING,
                    'type': 'test_service_type',
                },
                'name': 'baz',
                'instance': {
                    '?': {
                        'id': 'test_alt_instance_id',
                        'type': 'test_instance_type',
                    },
                    'assignFloatingIp': False,
                    'name': 'qux',
                    'required_by': 'test_service_id'
                },
            }
        ]

        mock_environment = mock.Mock(
            id='test_env_id',
            status=consts.STATUS_ID_READY,
            services=fake_services)
        mock_environment.configure_mock(name='test_env_name')

        expected_env = {
            'id': 'test_env_id',
            'in_progress': False,
            'info_box': 'test_env_info',
            'name': 'test_env_name',
            'status': 'Deployed'
        }

        expected_node_ids = (
            'External_Network', 'test_service_id', 'test_instance_id',
            'test_alt_service_id', 'test_alt_instance_id')

        result = topology.render_d3_data(self.request, mock_environment)
        result = json.loads(result)
        self.assertIsInstance(result, dict)
        self.assertIn('environment', result)
        self.assertIn('nodes', result)

        for key, val in expected_env.items():
            self.assertEqual(val, result['environment'][key])

        for node_id in expected_node_ids:
            self.assertIn(node_id,
                          [node['id'] for node in result['nodes']])

    def test_render_d3_data_without_environment(self):
        self.assertIsNone(topology.render_d3_data(self.request, None))
        # Test without environment.services
        mock_env = mock.Mock(services=None)
        self.assertIsNone(topology.render_d3_data(self.request, mock_env))



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

from django.conf import settings
import yaml

from muranodashboard import api
from muranodashboard.common import cache
from muranodashboard.dynamic_ui import yaql_expression


def package_list(request, marker=None, filters=None, paginate=False,
                 page_size=20, sort_dir=None, limit=None):
    limit = limit or getattr(settings, 'PACKAGES_LIMIT', 100)
    filters = filters or {}

    if paginate:
        request_size = page_size + 1
    else:
        request_size = limit

    if marker:
        filters['marker'] = marker
    if sort_dir:
        filters['sort_dir'] = sort_dir

    client = api.muranoclient(request)

    packages_iter = client.packages.filter(limit=request_size,
                                           **filters)

    has_more_data = False
    if paginate:
        packages = list(itertools.islice(packages_iter, request_size))
        if len(packages) > page_size:
            packages.pop()
            has_more_data = True
    else:
        packages = list(packages_iter)

    return packages, has_more_data


def apps_that_inherit(request, fqn):
    glare = getattr(settings, 'MURANO_USE_GLARE', False)
    if not glare:
        return []
    apps = api.muranoclient(request).packages.filter(inherits=fqn)
    return apps


def app_by_fqn(request, fqn, catalog=True, version=None):
    kwargs = {'fqn': fqn, 'catalog': catalog}
    glare = getattr(settings, 'MURANO_USE_GLARE', False)
    if glare and version:
        kwargs['version'] = version
    apps = api.muranoclient(request).packages.filter(**kwargs)
    try:
        return next(apps)
    except StopIteration:
        return None


def make_loader_cls():
    class Loader(yaml.SafeLoader):
        pass

    def yaql_constructor(loader, node):
        value = loader.construct_scalar(node)
        return yaql_expression.YaqlExpression(value)

    # workaround for PyYAML bug: http://pyyaml.org/ticket/221
    resolvers = {}
    for k, v in yaml.SafeLoader.yaml_implicit_resolvers.items():
        resolvers[k] = v[:]
    Loader.yaml_implicit_resolvers = resolvers

    Loader.add_constructor(u'!yaql', yaql_constructor)
    Loader.add_implicit_resolver(
        u'!yaql', yaql_expression.YaqlExpression, None)

    return Loader


# Here are cached some data calls to api; note that not every package attribute
# getter should be cached - only immutable ones could be safely cached. E.g.,
# it would be a mistake to cache Application Name because it is mutable and can
# be changed in Manage -> Packages while cache is immutable (i.e. it
# its contents are obtained from the api only the first time).
@cache.with_cache('ui', 'ui.yaml')
def get_app_ui(request, app_id):
    return api.muranoclient(request).packages.get_ui(app_id, make_loader_cls())


@cache.with_cache('logo', 'logo.png')
def get_app_logo(request, app_id):
    return api.muranoclient(request).packages.get_logo(app_id)


@cache.with_cache('supplier_logo', 'supplier_logo.png')
def get_app_supplier_logo(request, app_id):
    return api.muranoclient(request).packages.get_supplier_logo(app_id)


def get_app_fqn(request, app_id):
    return get_package_details(request, app_id).fully_qualified_name


def get_service_name(request, app_id):
    return get_package_details(request, app_id).name


@cache.with_cache('package_details')
def get_package_details(request, app_id):
    return api.muranoclient(request).packages.get(app_id)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from muranodashboard.catalog import tabs
from openstack_dashboard.test import helpers


class TestLicenseTab(helpers.APITestCase):
    @mock.patch('muranodashboard.catalog.tabs.services')
    def test_license(self, mock_services):
        """Check that a license is returned."""

        # Fake the services.get_app_forms() call.
        m = mock.Mock()
        m.base_fields = {
            'license': mock.Mock(
                description='Lorem ipsum dolor sit '
                            'amet, consectetur adipiscing elit.')
        }
        mock_services.get_app_forms.return_value = [('', m)]

        # Fake an application object, needed when instantiating tabs.
        app = mock.Mock()
        app.id = 1

        group = tabs.ApplicationTabs(self.request, application=app)
        li = group.get_tabs()[2]

        # Should return the license description
        li._get_license()
        self.assertEqual(
            'Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
            li.app.license)

    @mock.patch('muranodashboard.catalog.tabs.services')
    def test_no_license(self, mock_services):
        """Check that no license is returned."""

        # Fake the services.get_app_forms() call.
        m = mock.Mock()
        m.base_fields = {}
        mock_services.get_app_forms.return_value = [('', m)]

        # Fake an application object, needed when instantiating tabs.
        app = mock.Mock()
        app.id = 1

        group = tabs.ApplicationTabs(self.request, application=app)
        li = group.get_tabs()[2]

        # Should return the license description
        li._get_license()
        self.assertEqual('', li.app.license)


class TestRequirementsTab(helpers.APITestCase):
    @mock.patch('muranodashboard.catalog.tabs.services')
    def test_requirements(self, mock_services):
        """Check that requirements are returned."""

        m = mock.Mock()
        m.base_fields = {
            'flavor': mock.Mock(requirements={
                'min_disk': 10,
                'min_vcpus': 2,
                'min_memory_mb': 2048,
                'max_disk': 25,
                'max_vcpus': 5,
                'max_memory_mb': 16000,
            })
        }
        mock_services.get_app_forms.return_value = [('', m)]

        app = mock.Mock()
        app.id = 1

        group = tabs.ApplicationTabs(self.request, application=app)
        r = group.get_tabs()[1]

        # Should return the requirements list used by the template file.
        r._get_requirements()

        self.assertIn('Instance flavor:', r.app.requirements)
        flavor_req = r.app.requirements[1]

        self.assertIn('Minimum disk size: 10 GB',
                      flavor_req)
        self.assertIn('Minimum vCPUs: 2',
                      flavor_req)
        self.assertIn('Minimum RAM size: 2048 MB',
                      flavor_req)
        self.assertIn('Maximum disk size: 25 GB',
                      flavor_req)
        self.assertIn('Maximum vCPUs: 5',
                      flavor_req)
        self.assertIn('Maximum RAM size: 16000 MB',
                      flavor_req)

    @mock.patch('muranodashboard.catalog.tabs.services')
    def test_no_requirements(self, mock_services):
        """Check that no requirements are returned."""

        m = mock.Mock()
        m.base_fields = {}
        mock_services.get_app_forms.return_value = [('', m)]

        app = mock.Mock()
        app.id = 1

        group = tabs.ApplicationTabs(self.request, application=app)
        r = group.get_tabs()[1]

        # Should return an empty requirements list
        r._get_requirements()
        self.assertListEqual([], r.app.requirements)



#!/usr/bin/python
# Copyright (c) 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
import jinja2
import lxml.etree as et
import uuid
import sys
import os
import re

if not __name__ == "__main__":
    sys.exit(1)
if not len(sys.argv) >= 3:
    sys.exit(1)
if not os.path.exists(sys.argv[1]):
    sys.exit(1)

LOG_LINE_PATTERN = "^(?P<date>20[0-9]{2}\-[0-9]{2}\-[0-9]{2}) (?P<time>[0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]+) (?P<pid>[0-9]+) (?P<level>[A-Z]+) (?P<package>.*?) \[\-\](?P<log>.*?)$"


def get_attr(element, attr):
    return element.attrib[attr] if attr in element.attrib.keys() else None


def parse_log_file(log_file_path):
    LOG_RECORDS = []
    LAST_LOG_ENTRY = None

    with open(log_file_path, 'r') as log_file:
        for line in log_file.readlines():
            match = re.match(LOG_LINE_PATTERN, line, re.S)
            if match:
                LAST_LOG_ENTRY = {
                    'date': match.group('date'),
                    'time': match.group('time'),
                    'pid': match.group('pid'),
                    'level': match.group('level'),
                    'package': match.group('package'),
                    'log': match.group('log')
                }

                LOG_RECORDS.append(LAST_LOG_ENTRY)
            elif LAST_LOG_ENTRY is not None:
                LAST_LOG_ENTRY['log'] += line

    return [log_record for log_record in LOG_RECORDS if log_record['level'] == "ERROR"]


STATS = {
    'total': 0,
    'success': 0,
    'skip': 0,
    'error': 0,
    'failure': 0,
}

REPORT = {}

et.set_default_parser(et.XMLParser(huge_tree=True))

tree = et.parse(sys.argv[1])
root = tree.getroot()

STATS['total'] = int(root.attrib['tests'])
STATS['failure'] = int(root.attrib['failures'])
STATS['error'] = int(root.attrib['errors'])
STATS['skip'] = int(root.attrib['skip'])
STATS['unsuccess'] = STATS['failure'] + STATS['error'] + STATS['skip']
STATS['success'] = STATS['total'] - STATS['unsuccess']

for case in root:
    class_name = case.attrib['classname']

    screenshot_file = 'logs/artifacts/screenshots/%s.png' % case.attrib['name']
    screenshot_path = os.path.join(
        os.environ.get('WORKSPACE'),
        screenshot_file
    )

    test = {
        'name': case.attrib['name'],
        'time': case.attrib['time'],
        'result': 'success',
        'exc_type': None,
        'exc_message': None,
        'traceback': None,
        'output': case.text,
        'uuid': str(uuid.uuid1()),
        'screenshot': None
    }

    for child in case:
        test['exc_type'] = get_attr(child, 'type')
        test['exc_message'] = get_attr(child, 'message')
        test['traceback'] = child.text
        if child.tag == 'error':
            test['result'] = 'error'
            if os.path.exists(screenshot_path):
                test['screenshot'] = screenshot_file
        elif child.tag == 'failure':
            test['result'] = 'failure'
            if os.path.exists(screenshot_path):
                test['screenshot'] = screenshot_file
        elif child.tag == 'skipped':
            test['result'] = 'skip'

    if class_name not in REPORT.keys():
        REPORT[class_name] = {
            'tests': [],
            'stats': {
                'total': 0,
                'failure': 0,
                'error': 0,
                'skip': 0,
                'success': 0,
            },
            'result': 'success',
            'uuid': str(uuid.uuid1()),
        }

    REPORT[class_name]['tests'].append(test)
    REPORT[class_name]['stats']['total'] += 1
    REPORT[class_name]['stats'][test['result']] += 1

    TOTAL = REPORT[class_name]['stats']['total']

    for class_name in REPORT.keys():
        if REPORT[class_name]['stats']['failure'] > 0:
            REPORT[class_name]['result'] = 'failure'
        elif REPORT[class_name]['stats']['error'] > 0:
            REPORT[class_name]['result'] = 'failure'
        elif REPORT[class_name]['stats']['skip'] == TOTAL:
            REPORT[class_name]['result'] = 'skip'
        else:
            REPORT[class_name]['result'] = 'success'

jinja = jinja2.Environment(
    loader=jinja2.FileSystemLoader(os.path.join(
        os.path.dirname(__file__), 'templates')
    )
)

with open(sys.argv[2], 'w') as report_file:
    report_file.write(jinja.get_template(
        os.path.basename('report.template')
    ).render(
        report=REPORT,
        stats=STATS,
        coverage=False,
    ))



# Copyright (c) 2016 AT&T Corp
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core import exceptions
from django.core import validators as django_validator
from django import forms
from django.utils.translation import gettext_lazy as _
from unittest import mock

import unittest

from muranodashboard.dynamic_ui import fields


class TestFields(unittest.TestCase):

    def setUp(self):
        super(TestFields, self).setUp()
        self.request = mock.Mock()
        self.request.user.service_region = None
        self.addCleanup(mock.patch.stopall)
        mock.patch('horizon.utils.http.is_ajax', return_value=True).start()

    @mock.patch.object(fields, 'LOG')
    def test_fields_with_initial_request(self, mock_log):
        test_initial = {
            'request': 'foo_request',
            'foo': 'bar'
        }
        self._test_fields_decorator_with_initial_request(test_initial)
        mock_log.debug.assert_called_once_with(
            "Using 'request' value from initial dictionary")

    @fields.with_request
    def _test_fields_decorator_with_initial_request(self, request, **kwargs):
        self.assertEqual('foo_request', request)
        self.assertEqual({'foo': 'bar'}, kwargs)

    @mock.patch.object(fields, 'LOG')
    def test_fields_with_request(self, mock_log):
        test_request = {
            'foo': 'bar'
        }
        self._test_fields_decorator_with_request({}, request=test_request)
        mock_log.debug.assert_called_once_with("Using direct 'request' value")

    @fields.with_request
    def _test_fields_decorator_with_request(self, request, **kwargs):
        self.assertEqual({'foo': 'bar'}, request)
        self.assertEqual({}, kwargs)

    @mock.patch.object(fields, 'LOG')
    def test_fields_except_validation_error(self, mock_log):
        with self.assertRaises(forms.ValidationError) as cm:
            self._test_fields_decorator_with_validation_error({}, request=None)
        e = cm.exception
        self.assertEqual("Can't get a request information", e.message)
        mock_log.error.assert_called_once_with(
            "No 'request' value passed neither via initial dictionary, nor "
            "directly")

    @fields.with_request
    def _test_fields_decorator_with_validation_error(self, request, **kwargs):
        pass

    def test_make_yaql_validator(self):
        mock_validator_property = mock.MagicMock()
        mock_validator_property.__getitem__().spec.evaluate.return_value = True
        mock_validator_property.get.return_value = 'foo_message'

        validator_func = fields.make_yaql_validator(mock_validator_property)
        self.assertTrue(hasattr(validator_func, '__call__'))
        validator_func('bar')
        mock_validator_property.__getitem__().spec.evaluate.\
            assert_called_once_with(context=mock.ANY)

    def test_make_yaql_validator_except_validation_error(self):
        mock_validator_property = mock.MagicMock()
        mock_validator_property.__getitem__().spec.evaluate.return_value =\
            False
        mock_validator_property.get.return_value = 'foo_message'

        validator_func = fields.make_yaql_validator(mock_validator_property)
        self.assertTrue(hasattr(validator_func, '__call__'))
        with self.assertRaises(forms.ValidationError) as cm:
            validator_func('bar')
        e = cm.exception
        self.assertEqual('foo_message', e.message)

    def test_get_regex_validator(self):
        validator = django_validator.RegexValidator()
        test_expr = {
            'validators': [
                validator
            ]
        }
        result = fields.get_regex_validator(test_expr)
        self.assertEqual(validator, result)

    def test_get_regex_validator_except_error(self):
        for error in (TypeError, KeyError, IndexError):
            mock_expr = mock.MagicMock()
            mock_expr.__getitem__.side_effect = error
            result = fields.get_regex_validator(mock_expr)
            self.assertIsNone(result)

    def test_wrap_regex_validator(self):
        def _validator(value):
            pass

        func = fields.wrap_regex_validator(_validator, None)
        func(None)
        self.assertTrue(hasattr(func, '__call__'))

    def test_wrap_regex_validator_except_validation_error(self):
        def _validator(value):
            raise forms.ValidationError(None)

        with self.assertRaises(forms.ValidationError) as cm:
            func = fields.wrap_regex_validator(_validator, 'foo')
            func(None)
        e = cm.exception
        self.assertEqual('foo', e.message)

    @mock.patch.object(fields, 'glance')
    def test_get_murano_images(self, mock_glance):
        foo_image = mock.Mock(murano_property=None)
        foo_image.murano_image_info = '{"foo": "foo_val"}'
        bar_image = mock.Mock(murano_property=None)
        bar_image.murano_image_info = '{"bar": "bar_val"}'
        mock_glance.image_list_detailed.return_value = [
            [foo_image, bar_image], None
        ]

        murano_images = fields.get_murano_images(self.request)
        mock_glance.image_list_detailed.assert_called_once_with(self.request)
        self.assertEqual({"foo": "foo_val"}, foo_image.murano_property)
        self.assertEqual({"bar": "bar_val"}, bar_image.murano_property)

        expected_images = []
        foo_image.murano_property = {"foo": "foo_val"}
        bar_image.murano_property = {"bar": "bar_val"}
        expected_images.extend([foo_image, bar_image])

        self.assertEqual(expected_images, murano_images)

    @mock.patch.object(fields, 'exceptions')
    @mock.patch.object(fields, 'LOG')
    @mock.patch.object(fields, 'glance')
    def test_murano_images_except_exception(self, mock_glance, mock_log,
                                            mock_exceptions):
        mock_glance.image_list_detailed.side_effect = Exception

        murano_images = fields.get_murano_images(self.request)

        self.assertEqual([], murano_images)
        self.assertTrue(mock_log.error.called)
        mock_exceptions.handle.assert_called_once_with(
            self.request, _("Unable to retrieve public images."))

    @mock.patch.object(fields, 'messages')
    @mock.patch.object(fields, 'LOG')
    @mock.patch.object(fields, 'glance')
    def test_murano_images_except_value_error(self, mock_glance, mock_log,
                                              mock_messages):
        foo_image = mock.Mock(murano_property=None)
        foo_image.murano_image_info = "{'foo': 'foo_val'}"
        mock_glance.image_list_detailed.return_value = [
            [foo_image], None
        ]

        murano_images = fields.get_murano_images(self.request)

        self.assertEqual([], murano_images)
        mock_log.warning.assert_called_once_with(
            "JSON in image metadata is not valid. Check it in glance.")
        mock_messages.error.assert_called_once_with(
            self.request, _("Invalid murano image metadata"))

    def test_choice_get_title(self):
        choice = fields.Choice('test_title', True)
        self.assertEqual('test_title', fields._get_title(choice))
        self.assertIsNone(fields._get_title(None))

    def test_choice_disable_non_ready(self):
        choice = fields.Choice('test_title', True)
        self.assertEqual({}, fields._disable_non_ready(choice))
        choice = fields.Choice('test_title', False)
        self.assertEqual({'disabled': 'disabled'},
                         fields._disable_non_ready(choice))

    @mock.patch.object(fields, 'env_api')
    @mock.patch.object(fields, 'pkg_api')
    def test_make_select_cls_update(self, mock_pkg_api, mock_env_api):
        mock_pkg_api.app_by_fqn.return_value =\
            mock.Mock(fully_qualified_name='foo_class_fqn')
        mock_pkg_api.apps_that_inherit.return_value = [
            mock.Mock(fully_qualified_name='foo_class_fqn'),
            mock.Mock(fully_qualified_name='bar_class_fqn')
        ]
        expected_choices = [
            ('', 'Foo'), ('foo_app_id', 'foo_app_name'),
            ('bar_app_id', 'bar_app_name')
        ]

        foo_app = mock.MagicMock()
        foo_app.__getitem__.return_value = {'id': 'foo_app_id'}
        foo_app.configure_mock(name='foo_app_name')
        bar_app = mock.MagicMock()
        bar_app.__getitem__.return_value = {'id': 'bar_app_id'}
        bar_app.configure_mock(name='bar_app_name')
        mock_env_api.service_list_by_fqns.return_value = [foo_app, bar_app]

        dynamic_select_cls = fields.make_select_cls('foo_class_fqn')
        self.assertIsNotNone(dynamic_select_cls)
        self.assertEqual('DynamicSelect', dynamic_select_cls.__name__)

        dynamic_select = dynamic_select_cls(empty_value_message='Foo')
        dynamic_select.update({}, self.request, environment_id='foo_env_id')

        self.assertTrue(
            hasattr(dynamic_select.widget.add_item_link, '__call__'))
        self.assertEqual(expected_choices, dynamic_select.choices)
        self.assertIsNone(dynamic_select.initial)

        mock_pkg_api.app_by_fqn.assert_called_once_with(
            self.request, 'foo_class_fqn')
        mock_env_api.service_list_by_fqns.assert_called_once_with(
            self.request, 'foo_env_id',
            ['foo_class_fqn', 'bar_class_fqn']
        )

    @mock.patch.object(fields, 'env_api')
    @mock.patch.object(fields, 'pkg_api')
    def test_make_select_cls_update_2_choices(self, mock_pkg_api,
                                              mock_env_api):
        mock_pkg_api.app_by_fqn.return_value =\
            mock.Mock(fully_qualified_name='foo_class_fqn')
        mock_pkg_api.apps_that_inherit.return_value = []
        expected_choices = [
            ('', 'Foo'), ('foo_app_id', 'foo_app_name')
        ]

        foo_app = mock.MagicMock()
        foo_app.__getitem__.return_value = {'id': 'foo_app_id'}
        foo_app.configure_mock(name='foo_app_name')
        mock_env_api.service_list_by_fqns.return_value = [foo_app]

        dynamic_select_cls = fields.make_select_cls('foo_class_fqn')
        dynamic_select = dynamic_select_cls(empty_value_message='Foo')
        dynamic_select.update({}, self.request, environment_id='foo_env_id')

        self.assertEqual(expected_choices, dynamic_select.choices)
        self.assertEqual('foo_app_id', dynamic_select.initial)

        mock_pkg_api.app_by_fqn.assert_called_once_with(
            self.request, 'foo_class_fqn')
        mock_env_api.service_list_by_fqns.assert_called_once_with(
            self.request, 'foo_env_id', ['foo_class_fqn']
        )

    @mock.patch.object(fields, 'env_api')
    @mock.patch.object(fields, 'pkg_api')
    def test_make_select_cls_update_no_matching_classes(self, mock_pkg_api,
                                                        mock_env_api):
        mock_pkg_api.app_by_fqn.return_value = None
        mock_pkg_api.apps_that_inherit.return_value = []
        mock_env_api.service_list_by_fqns.return_value = []
        expected_choices = [('', 'Foo')]

        dynamic_select_cls = fields.make_select_cls('foo_class_fqn')
        dynamic_select = dynamic_select_cls(empty_value_message='Foo')
        dynamic_select.update({}, self.request, environment_id='foo_env_id')

        self.assertEqual(expected_choices, dynamic_select.choices)
        self.assertIsNone(dynamic_select.initial)

        mock_pkg_api.app_by_fqn.assert_called_once_with(
            self.request, 'foo_class_fqn')
        mock_env_api.service_list_by_fqns.assert_called_once_with(
            self.request, 'foo_env_id', [])

    @mock.patch.object(fields, 'reverse')
    @mock.patch.object(fields, 'env_api')
    @mock.patch.object(fields, 'pkg_api')
    def test_make_select_cls_update_make_link(self, mock_pkg_api, mock_env_api,
                                              mock_reverse):
        mock_pkg_api.app_by_fqn.return_value = None
        mock_pkg_api.apps_that_inherit.return_value = []
        mock_env_api.service_list_by_fqns.return_value = []
        mock_reverse.return_value = 'foo_url'

        dynamic_select_cls = fields.make_select_cls('foo_class_fqn')
        dynamic_select = dynamic_select_cls(empty_value_message='Foo')
        dynamic_select.update({}, self.request, environment_id='foo_env_id')

        result = dynamic_select.widget.add_item_link()
        self.assertEqual('', result)

        mock_pkg = mock.Mock(fully_qualified_name='foo_class_fqn')
        mock_pkg.configure_mock(name='foo_class_name')
        mock_pkg_api.app_by_fqn.return_value = mock_pkg
        dynamic_select.update({}, self.request, environment_id='foo_env_id')

        result = dynamic_select.widget.add_item_link()
        expected = '[["foo_class_name", "foo_url"]]'
        self.assertEqual(expected, result)

    @mock.patch.object(fields, 'env_api')
    @mock.patch.object(fields, 'pkg_api')
    def test_update_clean(self, mock_pkg_api, mock_env_api):
        mock_pkg_api.app_by_fqn.return_value = None
        mock_pkg_api.apps_that_inherit.return_value = []
        mock_env_api.service_list_by_fqns.return_value = []

        dynamic_select_cls = fields.make_select_cls('foo_class_fqn')
        dynamic_select = dynamic_select_cls(empty_value_message='Foo')
        dynamic_select.form = mock.Mock()
        dynamic_select.required = False
        dynamic_select.choices = [('value', '')]

        self.assertEqual('value', dynamic_select.clean('value'))


class TestRawProperty(unittest.TestCase):

    def test_finalize(self):
        class Control(object):
            def __init__(self):
                self.value = None

            @property
            def prop(self):
                return self.value

            @prop.setter
            def prop(self, value):
                self.value = value

            @prop.deleter
            def prop(self):
                delattr(self, 'value')

        mock_service = mock.Mock()
        mock_service.get_data.side_effect = ['foo_value']

        raw_property = fields.RawProperty('prop', 'foo_spec')
        props = raw_property.finalize(
            'foo_form_name', mock_service, Control)

        ctl = Control()

        result = props.fget(ctl)
        self.assertEqual('foo_value', result)

        props.fset(ctl, 'bar_value')
        self.assertEqual('bar_value', ctl.prop)

        props.fdel(ctl)
        self.assertNotIn('prop', ctl.__dict__)


class TestCustomPropertiesField(unittest.TestCase):

    def setUp(self):
        super(TestCustomPropertiesField, self).setUp()

        test_validator_1 = mock.MagicMock(__call__=lambda: None)
        test_validator_2 = {
            'expr': {
                'validators': [django_validator.RegexValidator()]
            }
        }
        test_validator_3 = {
            'expr': fields.RawProperty(None, None)
        }
        kwargs = {
            'validators': [
                test_validator_1, test_validator_2, test_validator_3
            ]
        }
        for arg in fields.FIELD_ARGS_TO_ESCAPE:
            kwargs[arg] = 'foo_' + arg

        custom_props_field = fields.CustomPropertiesField(**kwargs)

        for arg in fields.FIELD_ARGS_TO_ESCAPE:
            self.assertTrue(hasattr(custom_props_field, arg))
            self.assertEqual('foo_{0}'.format(arg),
                             getattr(custom_props_field, arg))
        self.assertEqual(3, len(custom_props_field.validators))

    def test_clean(self):
        mock_form = mock.Mock()
        mock_form.cleaned_data = 'test_cleaned_data'
        custom_props_field = fields.CustomPropertiesField()
        custom_props_field.form = mock_form

        custom_props_field.enabled = True
        self.assertEqual('foo', custom_props_field.clean('foo'))

        custom_props_field.enabled = False
        self.assertEqual('foo', custom_props_field.clean('foo'))

    def test_finalize_properties(self):
        finalize_properties = fields.CustomPropertiesField.finalize_properties
        kwargs = {
            'foo_raw_property': fields.RawProperty('foo_key', 'foo_spec')
        }
        mock_service = mock.Mock()

        result = finalize_properties(kwargs, 'foo_form_name', mock_service)
        self.assertIsNotNone(result)
        result = finalize_properties({}, 'foo_form_name', mock_service)
        self.assertIsNotNone(result)


class TestPasswordField(unittest.TestCase):

    def setUp(self):
        super(TestPasswordField, self).setUp()
        self.password_field = fields.PasswordField(None)
        self.password_field.original = True
        self.password_field.required = True
        self.addCleanup(mock.patch.stopall)

    def test_get_clone_name(self):
        self.assertEqual('foo-clone',
                         fields.PasswordField.get_clone_name('foo'))

    def test_compare(self):
        test_form_data = {'name': 'foo', 'name-clone': 'foo'}
        result = self.password_field.compare('name', test_form_data)
        self.assertIsNone(result)

    def test_compare_except_validation_error(self):
        test_form_data = {'name': 'foo', 'name-clone': 'bar'}
        self.assertRaises(forms.ValidationError, self.password_field.compare,
                          'name', test_form_data)

    def test_deepcopy(self):
        self.password_field.error_messages = {}
        test_memo = {}
        result = self.password_field.__deepcopy__(test_memo)
        self.assertIsInstance(result, fields.PasswordField)
        self.assertGreater(len(test_memo.keys()), 0)

        self.password_field.error_messages = ['foo_error', 'bar_error']
        test_memo = {}
        result = self.password_field.__deepcopy__(test_memo)
        self.assertIsInstance(result, fields.PasswordField)
        self.assertGreater(len(test_memo.keys()), 0)
        self.assertEqual(['foo_error', 'bar_error'], result.error_messages)

    def test_clone_field(self):
        self.assertFalse(self.password_field.has_clone)

        result = self.password_field.clone_field()
        self.assertIsInstance(result, fields.PasswordField)
        self.assertFalse(result.original)
        self.assertEqual('Confirm password', result.label)
        self.assertEqual('Please confirm your password',
                         result.error_messages['required'])
        self.assertEqual('Retype your password', result.help_text)


class TestFlavorChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestFlavorChoiceField, self).setUp()

        self.requirements = {
            'min_vcpus': 1,
            'min_disk': 100,
            'min_memory_mb': 500,
            'max_vcpus': 5,
            'max_disk': 5000,
            'max_memory_mb': 16000
        }
        kwargs = {
            'requirements': self.requirements
        }
        self.flavor_choice_field = fields.FlavorChoiceField(**kwargs)
        self.flavor_choice_field.choices = []
        self.flavor_choice_field.initial = None
        self.assertEqual(kwargs['requirements'],
                         self.flavor_choice_field.requirements)

        self.request = {'request': mock.Mock()}
        self.tiny_flavor = mock.Mock()
        self.tiny_flavor.configure_mock(id='id1', name='m1.tiny')
        self.small_flavor = mock.Mock()
        self.small_flavor.configure_mock(id='id2', name='m1.small')
        self.medium_flavor = mock.Mock()
        self.medium_flavor.configure_mock(id='id3', name='m1.medium')

        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'nova')
    def test_update(self, mock_nova):
        """"Test if flavor with any invalid requirement is excluded."""
        mock_nova.novaclient().flavors.list.return_value = [
            self.tiny_flavor, self.small_flavor, self.medium_flavor
        ]
        expected_choices = [
            ('id3', 'm1.medium'), ('id2', 'm1.small')
        ]
        valid_requirements = [
            ('vcpus', 2), ('disk', 101), ('ram', 501)
        ]
        invalid_requirements = [
            ('vcpus', 0), ('vcpus', 6), ('disk', 99), ('disk', 5001),
            ('ram', 499), ('ram', 16001)
        ]

        for req in valid_requirements:
            for flavor in (self.small_flavor, self.medium_flavor):
                setattr(flavor, req[0], req[1])

        for invalid_req in invalid_requirements:
            for valid_req in valid_requirements:
                if invalid_req[0] != valid_req[0]:
                    setattr(self.tiny_flavor, valid_req[0], valid_req[1])
            setattr(self.tiny_flavor, invalid_req[0], invalid_req[1])
            self.flavor_choice_field.update(self.request)
            self.assertEqual(expected_choices,
                             self.flavor_choice_field.choices)
            self.assertEqual('id3', self.flavor_choice_field.initial)

    @mock.patch.object(fields, 'nova')
    def test_update_without_requirements(self, mock_nova):
        mock_nova.novaclient().flavors.list.return_value = [
            self.tiny_flavor, self.small_flavor, self.medium_flavor
        ]
        del self.flavor_choice_field.requirements

        expected_choices = [
            ('id3', 'm1.medium'),
            ('id2', 'm1.small'),
            ('id1', 'm1.tiny')
        ]

        self.flavor_choice_field.update(self.request)
        self.assertEqual(expected_choices, self.flavor_choice_field.choices)
        self.assertEqual('id3', self.flavor_choice_field.initial)


class TestKeyPairChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestKeyPairChoiceField, self).setUp()
        self.request = {'request': mock.Mock()}
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'nova')
    def test_update(self, mock_nova):
        foo_keypair = mock.Mock()
        bar_keypair = mock.Mock()
        foo_keypair.configure_mock(name='foo')
        bar_keypair.configure_mock(name='bar')
        mock_nova.novaclient().keypairs.list.return_value = [
            foo_keypair, bar_keypair
        ]
        key_pair_choice_field = fields.KeyPairChoiceField()
        key_pair_choice_field.choices = []
        key_pair_choice_field.update(self.request)

        expected_choices = [
            ('', _('No keypair')), ('foo', 'foo'), ('bar', 'bar')
        ]
        self.assertEqual(sorted(expected_choices),
                         sorted(key_pair_choice_field.choices))


class TestSecurityGroupChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestSecurityGroupChoiceField, self).setUp()
        self.request = {'request': mock.Mock()}
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'neutron')
    def test_update(self, mock_neutron):
        mock_neutron.security_group_list.return_value = [
            mock.Mock(name_or_id='foo'),
            mock.Mock(name_or_id='bar')
        ]
        security_group_choice_field = fields.SecurityGroupChoiceField()
        security_group_choice_field.choices = []
        security_group_choice_field.update(self.request)

        expected_choices = [
            ('', _('Application default security group')),
            ('foo', 'foo'), ('bar', 'bar')
        ]
        self.assertEqual(sorted(expected_choices),
                         sorted(security_group_choice_field.choices))


class TestImageChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestImageChoiceField, self).setUp()
        self.request = {'request': mock.Mock()}
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'get_murano_images')
    def test_update(self, mock_get_murano_images):
        mock_get_murano_images.return_value = [
            # Test successful control flow.
            mock.Mock(id='foo_image_id', murano_property={
                      'title': 'foo_image_title', 'type': 'png'},
                      status='active'),
            # Test whether second continue statement works.
            mock.Mock(id='bar_image_id', murano_property={
                      'title': 'bar_image_title', 'type': 'png'},
                      status='active'),
            mock.Mock(id='baz_image_id', murano_property={
                      'title': 'baz_image_title', 'type': 'jpg'},
                      status='active')
        ]
        image_choice_field = fields.ImageChoiceField()
        image_choice_field.image_type = 'png'
        image_choice_field.choices = []
        image_choice_field.update(self.request)

        self.assertEqual(("", _("Select Image")),
                         image_choice_field.choices[0])
        self.assertEqual("bar_image_id", image_choice_field.choices[1][0])
        self.assertIsInstance(image_choice_field.choices[1][1], fields.Choice)

        # Test that 'Select Image' is dropped if only one result
        image_choice_field.image_type = 'jpg'
        image_choice_field.choices = []
        image_choice_field.update(self.request)

        self.assertEqual('baz_image_id', image_choice_field.choices[0][0])
        self.assertIsInstance(image_choice_field.choices[0][1], fields.Choice)

        # Test whether first continue statement works.
        mock_get_murano_images.return_value = [
            mock.Mock(murano_property={
                      'title': 'bar_image_title', 'type': None},
                      status=None)
        ]
        image_choice_field.image_type = ''
        image_choice_field.choices = []
        image_choice_field.update(self.request)
        expected_choices = [("", _("No images available"))]
        self.assertEqual(expected_choices, image_choice_field.choices)


class TestNetworkChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestNetworkChoiceField, self).setUp()
        self.network_choice_field = fields.NetworkChoiceField(
            filter=None,
            murano_networks='exclude',
            allow_auto=True)
        self.request = {'request': mock.Mock()}
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'net')
    def test_update(self, mock_net):
        mock_net.get_available_networks.return_value = [
            (('foo', 'foo'), _('Foo'))
        ]
        expected_choices = [
            ((None, None), _('Auto')), (('foo', 'foo'), _('Foo'))
        ]

        self.network_choice_field.update(self.request)
        self.assertEqual(expected_choices, self.network_choice_field.choices)
        mock_net.get_available_networks.assert_called_once_with(
            self.request['request'], None, 'exclude')

    def test_to_python(self):
        self.assertEqual({'foo': 'bar'},
                         self.network_choice_field.to_python('{"foo": "bar"}'))
        self.assertEqual((None, None),
                         self.network_choice_field.to_python(None))


class TestVolumeChoiceField(unittest.TestCase):

    def setUp(self):
        super(TestVolumeChoiceField, self).setUp()
        self.request = {'request': mock.Mock()}
        self.addCleanup(mock.patch.stopall)

    @mock.patch.object(fields, 'cinder')
    def test_update(self, mock_cinder):
        foo_vol = mock.Mock()
        bar_snap = mock.Mock()
        baz_snap = mock.Mock()
        foo_vol.configure_mock(name='foo_vol', id='foo_id', status='available')
        bar_snap.configure_mock(name='bar_snap', id='bar_id',
                                status='available')
        baz_snap.configure_mock(name='baz_snap', id='baz_id', status='error')
        mock_cinder.volume_list.return_value = [foo_vol]
        mock_cinder.volume_snapshot_list.return_value = [bar_snap]
        volume_choice_field = fields.VolumeChoiceField()
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('Select volume')), ('foo_id', 'foo_vol'),
            ('bar_id', 'bar_snap')
        ]

        self.assertEqual(sorted(expected_choices),
                         sorted(volume_choice_field.choices))

    @mock.patch.object(fields, 'cinder')
    def test_update_withoutsnapshot(self, mock_cinder):
        foo_vol = mock.Mock()
        bar_vol = mock.Mock()
        baz_snap = mock.Mock()
        foo_vol.configure_mock(name='foo_vol', id='foo_id', status='available')
        bar_vol.configure_mock(name='bar_vol', id='bar_id', status='error')
        baz_snap.configure_mock(name='baz_snap', id='baz_id',
                                status='available')
        mock_cinder.volume_list.return_value = [foo_vol]
        mock_cinder.volume_snapshot_list.return_value = [baz_snap]
        volume_choice_field = fields.VolumeChoiceField(include_snapshots=False)
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('Select volume')), ('foo_id', 'foo_vol')
        ]

        self.assertEqual(sorted(expected_choices),
                         sorted(volume_choice_field.choices))

    @mock.patch.object(fields, 'cinder')
    def test_update_withoutvolume(self, mock_cinder):
        foo_vol = mock.Mock()
        baz_snap = mock.Mock()
        foo_vol.configure_mock(name='foo_vol', id='foo_id', status='available')
        baz_snap.configure_mock(name='baz_snap', id='baz_id',
                                status='available')
        mock_cinder.volume_list.return_value = [foo_vol]
        mock_cinder.volume_snapshot_list.return_value = [baz_snap]
        volume_choice_field = fields.VolumeChoiceField(include_volumes=False)
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('Select volume')), ('baz_id', 'baz_snap')
        ]

        self.assertEqual(sorted(expected_choices),
                         sorted(volume_choice_field.choices))

    @mock.patch.object(fields, 'exceptions')
    @mock.patch.object(fields, 'cinder')
    def test_update_except_snapshot_list_exception(self, mock_cinder,
                                                   mock_exceptions):
        foo_vol = mock.Mock()
        bar_vol = mock.Mock()
        foo_vol.configure_mock(name='foo_vol', id='foo_id', status='available')
        bar_vol.configure_mock(name='bar_vol', id='bar_id', status='error')
        mock_cinder.volume_list.return_value = [foo_vol]
        mock_cinder.volume_snapshot_list.side_effect = Exception
        volume_choice_field = fields.VolumeChoiceField(include_volumes=True,
                                                       include_snapshots=True)
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('Select volume')), ('foo_id', 'foo_vol')
        ]

        self.assertEqual(sorted(expected_choices),
                         sorted(volume_choice_field.choices))
        mock_exceptions.handle.assert_called_once_with(
            self.request['request'], _('Unable to retrieve snapshot list.'))

    @mock.patch.object(fields, 'exceptions')
    @mock.patch.object(fields, 'cinder')
    def test_update_except_volume_list_exception(self, mock_cinder,
                                                 mock_exceptions):
        bar_snap = mock.Mock()
        bar_snap.configure_mock(name='bar_snap', id='bar_id',
                                status='available')
        mock_cinder.volume_list.side_effect = Exception
        mock_cinder.volume_snapshot_list.return_value = [bar_snap]
        volume_choice_field = fields.VolumeChoiceField(include_volumes=True,
                                                       include_snapshots=True)
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('Select volume')), ('bar_id', 'bar_snap')
        ]

        self.assertEqual(expected_choices, volume_choice_field.choices)
        mock_exceptions.handle.assert_called_once_with(
            self.request['request'], _('Unable to retrieve volume list.'))

    @mock.patch.object(fields, 'exceptions')
    @mock.patch.object(fields, 'cinder')
    def test_update_except_exception(self, mock_cinder, mock_exceptions):
        mock_cinder.volume_list.side_effect = Exception
        mock_cinder.volume_snapshot_list.side_effect = Exception
        volume_choice_field = fields.VolumeChoiceField(include_volumes=True,
                                                       include_snapshots=True)
        volume_choice_field.choices = []
        volume_choice_field.update(self.request)

        expected_choices = [
            ('', _('No volumes available'))
        ]
        expected_calls = [
            mock.call(self.request['request'],
                      _('Unable to retrieve volume list.')),
            mock.call(self.request['request'],
                      _('Unable to retrieve snapshot list.'))
        ]
        self.assertEqual(expected_choices, volume_choice_field.choices)
        mock_exceptions.handle.assert_has_calls(expected_calls)


class TestAZoneChoiceField(unittest.TestCase):

    @mock.patch.object(fields, 'nova')
    def test_update(self, mock_nova):
        mock_nova.novaclient().availability_zones.list.return_value = [
            mock.Mock(zoneName='foo_zone', zoneState={'available': True}),
            mock.Mock(zoneName='bar_zone', zoneState={'available': True}),
            mock.Mock(zoneName='baz_zone', zoneState={'available': False})
        ]
        request = {'request': mock.Mock()}
        a_zone_choice_field = fields.AZoneChoiceField()
        a_zone_choice_field.choices = []

        expected_choices = [
            ("bar_zone", "bar_zone"), ("foo_zone", "foo_zone")
        ]
        a_zone_choice_field.update(request)
        self.assertEqual(expected_choices, a_zone_choice_field.choices)

    @mock.patch.object(fields, 'exceptions')
    @mock.patch.object(fields, 'nova')
    def test_update_except_exception(self, mock_nova, mock_exc):
        mock_nova.novaclient().availability_zones.list.side_effect = Exception
        request = {'request': mock.Mock()}
        a_zone_choice_field = fields.AZoneChoiceField()
        a_zone_choice_field.choices = []

        expected_choices = [
            ("", _("No availability zones available"))
        ]
        a_zone_choice_field.update(request)
        self.assertEqual(expected_choices, a_zone_choice_field.choices)
        mock_exc.handle.assert_called_once_with(request['request'], mock.ANY)


class TestBooleanField(unittest.TestCase):

    def test_boolean_field(self):
        class Widget(object):
            def __init__(self, attrs):
                self.attrs = attrs

        boolean_field = fields.BooleanField(widget=Widget)
        self.assertIsInstance(boolean_field.widget, Widget)
        self.assertEqual({'class': 'checkbox'}, boolean_field.widget.attrs)
        self.assertFalse(boolean_field.required)

        boolean_field = fields.BooleanField()
        self.assertIsInstance(boolean_field.widget, forms.CheckboxInput)
        self.assertEqual({'class': 'checkbox'}, boolean_field.widget.attrs)
        self.assertFalse(boolean_field.required)


class TestDatabaseListField(unittest.TestCase):

    def setUp(self):
        super(TestDatabaseListField, self).setUp()
        self.database_list_field = fields.DatabaseListField()
        self.addCleanup(mock.patch.stopall)

    def test_to_python(self):
        self.assertEqual([], self.database_list_field.to_python(None))
        self.assertEqual(['foo', 'bar'],
                         self.database_list_field.to_python('foo ,bar '))

    def test_validate(self):
        valid_value = ['a123', '_123', 'a123_$#@']
        result = self.database_list_field.validate(valid_value)
        self.assertIsNone(result)

    def test_validate_except_validation_error(self):
        invalid_value = ['123abc']

        expected_error = "First symbol should be latin letter or underscore. "\
                         "Subsequent symbols can be latin letter, numeric, "\
                         "underscore, at sign, number sign or dollar sign"
        with self.assertRaises(exceptions.ValidationError) as cm:
            self.database_list_field.validate(invalid_value)
        e = cm.exception
        self.assertEqual(expected_error, e.message)


class TestErrorWidget(unittest.TestCase):

    def test_render(self):
        error_widget = fields.ErrorWidget()
        error_widget.message = 'foo_message'
        result = error_widget.render("'foo_name'", None)
        self.assertEqual("<div name='foo_name'>foo_message</div>", result)



#    Copyright (c) 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext_lazy
from horizon import exceptions
from horizon import tables
from openstack_dashboard.api import glance
from openstack_dashboard import policy

from muranodashboard.common import utils as md_utils


class MarkImage(tables.LinkAction):
    name = "mark_image"
    verbose_name = _("Mark Image")
    url = "horizon:app-catalog:images:mark_image"
    classes = ("ajax-modal",)
    icon = "plus"
    policy_rules = (("murano", "mark_image"),)


class RemoveImageMetadata(policy.PolicyTargetMixin, tables.DeleteAction):
    policy_rules = (("murano", "remove_image_metadata"),)

    @staticmethod
    def action_present(count):
        return ngettext_lazy(
            u"Delete Metadata",
            u"Delete Metadata",
            count
        )

    @staticmethod
    def action_past(count):
        return ngettext_lazy(
            u"Deleted Metadata",
            u"Deleted Metadata",
            count
        )

    def delete(self, request, obj_id):
        try:
            remove_props = ['murano_image_info']
            glance.image_update_properties(request, obj_id, remove_props)
        except Exception:
            exceptions.handle(request, _('Unable to remove metadata'),
                              redirect=reverse(
                                  'horizon:app-catalog:images:index'))


class MarkedImagesTable(tables.DataTable):
    image = tables.Column(
        'name',
        link='horizon:project:images:images:detail',
        verbose_name=_('Image')
    )
    type = tables.Column(lambda obj: getattr(obj, 'type', None),
                         verbose_name=_('Type'))
    title = md_utils.Column(lambda obj: getattr(obj, 'title', None),
                            verbose_name=_('Title'))

    class Meta(object):
        name = 'marked_images'
        verbose_name = _('Marked Images')
        table_actions = (MarkImage, RemoveImageMetadata)
        row_actions = (RemoveImageMetadata,)



# Copyright (c) 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from muranodashboard.dynamic_ui import version


class TestVersions(unittest.TestCase):
    def setUp(self):
        super(TestVersions, self).setUp()
        self.original = version.LATEST_FORMAT_VERSION
        version.LATEST_FORMAT_VERSION = '2.4'

    def tearDown(self):
        version.LATEST_FORMAT_VERSION = self.original
        super(TestVersions, self).tearDown()

    def test_get_latest_as_semver(self):
        latest = version.get_latest_version()
        self.assertEqual(2, latest.major)
        self.assertEqual(4, latest.minor)
        self.assertEqual(0, latest.patch)

    def test_exact_match(self):
        version.check_version('2.4')

    def test_older_in_family(self):
        version.check_version('2.1')

    def test_oldest_in_family(self):
        version.check_version('2')

    def test_newer(self):
        self.assertRaises(ValueError, version.check_version, '2.5')

    def test_uncompatible_old(self):
        self.assertRaises(ValueError, version.check_version, '1.6')

    def test_uncompatible_new(self):
        self.assertRaises(ValueError, version.check_version, '3.0')



# Copyright (c) 2016 AT&T Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
import unittest
from unittest import mock

from django.conf import settings
from django.utils.translation import gettext_lazy as _

from muranoclient.common import exceptions as exc
from muranodashboard.environments import tables
from muranodashboard.environments import tabs


class TestOverviewTab(unittest.TestCase):

    def setUp(self):
        super(TestOverviewTab, self).setUp()
        self.overview_tab = tabs.OverviewTab(None)

        self.assertEqual(_('Component'), self.overview_tab.name)
        self.assertEqual('_service', self.overview_tab.slug)
        self.assertEqual('services/_overview.html',
                         self.overview_tab.template_name)

    @mock.patch.object(tabs, 'heat_api')
    @mock.patch.object(tabs, 'nova_api')
    @mock.patch.object(tabs, 'consts')
    def test_get_context_data(self, mock_consts, mock_nova_api, mock_heat_api):
        mock_consts.STATUS_DISPLAY_CHOICES = [('foo_status_id', 'foo_status')]
        foo_mock_instance = mock.Mock(id='foo_instance_id')
        foo_mock_instance.configure_mock(name='foo-instance-name')
        bar_mock_instance = mock.Mock(id='bar_instance_id')
        bar_mock_instance.configure_mock(name='bar-instance-name')
        baz_mock_instance = mock.Mock(id='baz_instance_id')
        baz_mock_instance.configure_mock(name='baz-instance-name')
        mock_nova_api.server_list.side_effect = [
            ([foo_mock_instance], False), ([bar_mock_instance], False),
            ([baz_mock_instance], False)
        ]
        mock_heat_api.stacks_list.side_effect = [
            (
                [mock.Mock(id='foo_stack_id', stack_name='foo_stack_name')],
                False, False
            ),
            (
                [mock.Mock(id='bar_stack_id', stack_name='bar_stack_name')],
                False, False
            ),
            (
                [mock.Mock(id='baz_stack_id', stack_name='baz_stack_name')],
                False, False
            )
        ]
        mock_request = mock.Mock()
        mock_request.session = {'django_timezone': 'UTC'}

        expected_service = {
            'service': collections.OrderedDict([
                ('Name', 'foo_service_data_name'),
                ('ID', 'foo_service_data_id'),
                ('Type', 'Unknown'),
                ('Status', 'foo_status'),
                ('Domain', 'foo_domain'),
                ('Application repository', 'foo_repository'),
                ('Load Balancer URI', 'foo_uri'),
                ('Floating IP', 'foo_floatingip'),
                ('Instance',
                    {'name': 'foo-instance-name', 'id': 'foo_instance_id'}),
                ('Stack',
                    {'id': 'foo_stack_id', 'name': 'foo_stack_name'}),
                ('Instances', [
                    {'name': 'bar-instance-name', 'id': 'bar_instance_id'},
                    {'name': 'baz-instance-name', 'id': 'baz_instance_id'}]),
                ('Stacks', [
                    {'id': 'bar_stack_id', 'name': 'bar_stack_name'},
                    {'id': 'baz_stack_id', 'name': 'baz_stack_name'}])
            ])
        }

        def service_data_side_effect(*args, **kwargs):
            if args[0] == 'instances':
                return [
                    {
                        'status': 'bar_status_id',
                        'id': 'bar_service_data_id',
                        'name': 'instance-name',
                        'openstackId': 'bar_instance_id'
                    },
                    {
                        'status': 'baz_status_id',
                        'id': 'baz_service_data_id',
                        'name': 'instance-name',
                        'openstackId': 'baz_instance_id'
                    }
                ]
            elif args[0] == 'instance':
                return {
                    'name': 'instance-name',
                    'openstackId': 'instance_id'
                }
            elif args[0] == '?':
                return {
                    'status': 'foo_status_id',
                    'id': 'foo_service_data_id',
                }

        service_data = mock.MagicMock()
        service_data.__getitem__.side_effect = service_data_side_effect
        service_data.configure_mock(name='foo_service_data_name')
        service_data.domain = 'foo_domain'
        service_data.repository = 'foo_repository'
        service_data.uri = 'foo_uri'
        service_data.floatingip = 'foo_floatingip'

        self.overview_tab.tab_group = mock.Mock()
        self.overview_tab.tab_group.kwargs = {
            'service': service_data
        }

        result = self.overview_tab.get_context_data(mock_request)

        self.assertIsInstance(result, dict)
        self.assertIn('service', result)
        self.assertEqual(expected_service, result)
        self.assertEqual(3, mock_nova_api.server_list.call_count)
        self.assertEqual(3, mock_heat_api.stacks_list.call_count)
        mock_nova_api.server_list.assert_any_call(mock_request)
        mock_heat_api.stacks_list.assert_any_call(mock_request, sort_dir='asc')

    @mock.patch.object(tabs, 'heat_api')
    @mock.patch.object(tabs, 'nova_api')
    def test_get_context_data_find_stack_has_more(self, mock_nova_api,
                                                  mock_heat_api):
        foo_mock_instance = mock.Mock(id='foo_instance_id')
        foo_mock_instance.configure_mock(name='foo-instance-name')
        mock_nova_api.server_list.side_effect = [
            ([foo_mock_instance], False)
        ]
        mock_heat_api.stacks_list.side_effect = [
            (
                [mock.Mock(id='bar_stack_id', stack_name='bar_stack_name')],
                True, False
            ),
            (
                [mock.Mock(id='foo_stack_id', stack_name='foo_stack_name')],
                False, False
            )
        ]
        mock_request = mock.Mock()

        expected_service = {
            'service': collections.OrderedDict([
                ('Name', 'foo_service_data_name'),
                ('ID', 'foo_service_data_id'),
                ('Type', 'Unknown'),
                ('Status', ''),
                ('Domain', 'Not in domain'),
                ('Application repository', 'foo_repository'),
                ('Load Balancer URI', 'foo_uri'),
                ('Floating IP', 'foo_floatingip'),
                ('Instance',
                    {'name': 'foo-instance-name', 'id': 'foo_instance_id'}),
                ('Stack',
                    {'id': 'foo_stack_id', 'name': 'foo_stack_name'}),
            ])
        }

        expected_mock_calls = [
            mock.call(mock_request, sort_dir='asc'),
            mock.call(mock_request, sort_dir='asc', marker='bar_stack_id')
        ]

        def service_data_side_effect(*args, **kwargs):
            if args[0] == 'instance':
                return {
                    'name': 'instance-name',
                    'openstackId': 'instance_id'
                }
            elif args[0] == '?':
                return {
                    'status': 'foo_status_id',
                    'id': 'foo_service_data_id',
                }

        service_data = mock.MagicMock()
        service_data.__getitem__.side_effect = service_data_side_effect
        service_data.configure_mock(name='foo_service_data_name')
        service_data.domain = None
        service_data.repository = 'foo_repository'
        service_data.uri = 'foo_uri'
        service_data.floatingip = 'foo_floatingip'

        self.overview_tab.tab_group = mock.Mock()
        self.overview_tab.tab_group.kwargs = {
            'service': service_data
        }

        result = self.overview_tab.get_context_data(mock_request)

        self.assertIsInstance(result, dict)
        self.assertIn('service', result)
        self.assertEqual(expected_service, result)
        # Test whether the expected number of calls were made. 1 call should
        # be made to nova_api but 2 should be made to heat_api, since that
        # call is recursive.
        self.assertEqual(1, mock_nova_api.server_list.call_count)
        self.assertEqual(2, mock_heat_api.stacks_list.call_count)
        mock_nova_api.server_list.assert_any_call(mock_request)
        self.assertEqual(expected_mock_calls,
                         mock_heat_api.stacks_list.mock_calls)


class TestServiceLogsTab(unittest.TestCase):

    def setUp(self):
        super(TestServiceLogsTab, self).setUp()
        self.service_logs_tab = tabs.ServiceLogsTab(None)

        self.assertEqual(_('Logs'), self.service_logs_tab.name)
        self.assertEqual('service_logs', self.service_logs_tab.slug)
        self.assertEqual('services/_logs.html',
                         self.service_logs_tab.template_name)
        self.assertFalse(self.service_logs_tab.preload)

    @mock.patch.object(tabs, 'api')
    def test_get_context_data(self, mock_api):
        mock_api.get_status_messages_for_service.return_value = ['foo_report']
        mock_request = mock.Mock()
        self.service_logs_tab.tab_group = mock.Mock()

        self.service_logs_tab.tab_group.kwargs = {
            'service_id': 'foo_service_id',
            'environment_id': 'foo_environment_id'
        }

        reports = self.service_logs_tab.get_context_data(mock_request)

        self.assertEqual({'reports': ['foo_report']}, reports)
        mock_api.get_status_messages_for_service.assert_called_once_with(
            mock_request, 'foo_service_id', 'foo_environment_id')


class TestEnvLogsTab(unittest.TestCase):

    def setUp(self):
        super(TestEnvLogsTab, self).setUp()
        self.env_logs_tab = tabs.EnvLogsTab(None)

        self.assertEqual(_('Logs'), self.env_logs_tab.name)
        self.assertEqual('env_logs', self.env_logs_tab.slug)
        self.assertEqual('deployments/_logs.html',
                         self.env_logs_tab.template_name)
        self.assertFalse(self.env_logs_tab.preload)

    def test_get_context_data(self):
        mock_report = mock.Mock(created='1970-01-01T12:34:00')
        self.env_logs_tab.tab_group = mock.Mock()
        self.env_logs_tab.tab_group.kwargs = {
            'logs': [mock_report]
        }
        mock_request = mock.MagicMock()
        mock_request.session = {'django_timezone': 'UTC'}

        reports = self.env_logs_tab.get_context_data(mock_request)

        mock_report.created = '1970-01-01 12:34:00'
        self.assertEqual({'reports': [mock_report]}, reports)


class TestLatestLogTab(unittest.TestCase):

    def test_allowed(self):
        mock_request = mock.MagicMock()
        mock_request.session = {'django_timezone': 'UTC'}
        mock_report = mock.Mock(created='1970-01-01T12:34:00')
        tab_group = mock.Mock()
        tab_group.kwargs = {'logs': [mock_report]}
        latest_logs_tab = tabs.LatestLogsTab(tab_group, request=mock_request)

        self.assertEqual(_('Latest Deployment Log'), latest_logs_tab.name)
        mock_report.created = '1970-01-01 12:34:00'
        self.assertEqual([mock_report], latest_logs_tab.allowed(mock_request))


class TestEnvConfigTab(unittest.TestCase):

    def setUp(self):
        super(TestEnvConfigTab, self).setUp()
        mock_tab_group = mock.Mock(kwargs={})
        mock_request = mock.Mock()
        self.env_config_tab = tabs.EnvConfigTab(mock_tab_group, mock_request)

        self.assertEqual(_('Configuration'), self.env_config_tab.name)
        self.assertEqual('env_config', self.env_config_tab.slug)
        self.assertEqual((tables.EnvConfigTable,),
                         self.env_config_tab.table_classes)
        self.assertEqual('horizon/common/_detail_table.html',
                         self.env_config_tab.template_name)
        self.assertFalse(self.env_config_tab.preload)

    def test_get_environment_and_configuration_data(self):
        self.env_config_tab.tab_group = mock.Mock()
        self.env_config_tab.tab_group.kwargs = {
            'deployment': {
                'services': ['foo_service']
            }
        }

        result = self.env_config_tab.get_environment_configuration_data()
        self.assertEqual(['foo_service'], result)


class TestEnvironmentTopologyTab(unittest.TestCase):

    def setUp(self):
        super(TestEnvironmentTopologyTab, self).setUp()
        self.env_topology_tab = tabs.EnvironmentTopologyTab(None)

        self.assertEqual(_('Topology'), self.env_topology_tab.name)
        self.assertEqual('topology', self.env_topology_tab.slug)
        self.assertEqual('services/_detail_topology.html',
                         self.env_topology_tab.template_name)
        self.assertFalse(self.env_topology_tab.preload)

    @mock.patch.object(tabs, 'api')
    def test_allowed_true(self, mock_api):
        self.env_topology_tab.tab_group = mock.Mock()
        self.env_topology_tab.tab_group.kwargs = {
            'environment_id': 'foo_env_id'
        }

        mock_api.load_environment_data.return_value =\
            '{"environment": {"status": "foo_status"}}'  # d3 data
        self.assertTrue(self.env_topology_tab.allowed(None))
        mock_api.load_environment_data.assert_called_with(None, 'foo_env_id')

    @mock.patch.object(tabs, 'api')
    def test_allowed_false(self, mock_api):
        self.env_topology_tab.tab_group = mock.Mock()
        self.env_topology_tab.tab_group.kwargs = {
            'environment_id': 'foo_env_id'
        }

        mock_api.load_environment_data.return_value =\
            '{"environment": {"status": null}}'  # d3 data
        self.assertFalse(self.env_topology_tab.allowed(None))
        mock_api.load_environment_data.assert_called_with(None, 'foo_env_id')


@mock.patch.object(tabs, 'api')
class TestEnvironmentServicesTab(unittest.TestCase):

    def setUp(self):
        super(TestEnvironmentServicesTab, self).setUp()

        test_kwargs = {'environment_id': 'foo_env_id'}
        mock_tab_group = mock.Mock(kwargs=test_kwargs)
        self.mock_request = mock.Mock()
        self.env_services_tab = tabs.EnvironmentServicesTab(mock_tab_group,
                                                            self.mock_request)

        self.assertEqual(_('Components'), self.env_services_tab.name)
        self.assertEqual('services', self.env_services_tab.slug)
        self.assertEqual((tables.ServicesTable,),
                         self.env_services_tab.table_classes)
        self.assertEqual('services/_service_list.html',
                         self.env_services_tab.template_name)
        self.assertFalse(self.env_services_tab.preload)

    def test_get_services_data(self, mock_api):
        mock_api.services_list.return_value = ['foo_service']

        services_data = self.env_services_tab.get_services_data()

        self.assertEqual(['foo_service'], services_data)
        self.assertEqual('foo_env_id', self.env_services_tab.environment_id)

    @mock.patch.object(tabs, 'reverse')
    @mock.patch.object(tabs, 'exceptions')
    def test_get_services_data_except_http_forbidden(
            self, mock_exc, mock_reverse, mock_api):
        mock_api.services_list.side_effect = exc.HTTPForbidden
        mock_reverse.return_value = 'foo_reverse_url'

        services_data = self.env_services_tab.get_services_data()

        self.assertEqual([], services_data)
        expected_msg = _(
            'Unable to retrieve list of services. This environment '
            'is deploying or already deployed by other user.')
        mock_exc.handle.assert_called_once_with(
            self.mock_request, expected_msg, redirect='foo_reverse_url')
        mock_reverse.assert_called_once_with(
            'horizon:app-catalog:environments:index')

    @mock.patch.object(tabs, 'reverse')
    @mock.patch.object(tabs, 'exceptions')
    def test_get_services_data_except_internal_server_and_not_found_errors(
            self, mock_exc, mock_reverse, mock_api):
        mock_reverse.return_value = 'foo_reverse_url'
        expected_msg = "Environment with id foo_env_id doesn't exist anymore"

        for exception_cls in (exc.HTTPInternalServerError, exc.HTTPNotFound):
            mock_api.services_list.side_effect = exception_cls

            services_data = self.env_services_tab.get_services_data()

            self.assertEqual([], services_data)
            mock_exc.handle.assert_called_with(
                self.mock_request, expected_msg, redirect='foo_reverse_url')
            mock_reverse.assert_called_with(
                'horizon:app-catalog:environments:index')

    @mock.patch.object(tabs, 'exceptions')
    def test_get_services_data_except_http_unauthorized(
            self, mock_exc, mock_api):
        mock_api.services_list.side_effect = exc.HTTPUnauthorized
        self.env_services_tab.get_services_data()
        mock_exc.handle.assert_called_once_with(self.mock_request)

    def test_get_context_data(self, _):
        setattr(settings, 'MURANO_USE_GLARE', True)

        expected_context = {
            'MURANO_USE_GLARE': True,
            'table': mock.ANY,
            'services_table': mock.ANY
        }

        context = self.env_services_tab.get_context_data(self.mock_request)

        for key, val in expected_context.items():
            self.assertEqual(val, context.get(key))


@mock.patch.object(tabs, 'api')
class TestDeploymentTab(unittest.TestCase):

    def setUp(self):
        super(TestDeploymentTab, self).setUp()

        test_kwargs = {'environment_id': 'foo_env_id'}
        mock_tab_group = mock.Mock(kwargs=test_kwargs)
        self.mock_request = mock.Mock()
        self.deployment_tab = tabs.DeploymentTab(mock_tab_group,
                                                 self.mock_request)

        self.assertEqual(_('Deployment History'), self.deployment_tab.name)
        self.assertEqual('deployments', self.deployment_tab.slug)
        self.assertEqual((tables.DeploymentsTable,),
                         self.deployment_tab.table_classes)
        self.assertEqual('horizon/common/_detail_table.html',
                         self.deployment_tab.template_name)
        self.assertFalse(self.deployment_tab.preload)

    @mock.patch.object(tabs, 'policy')
    def test_allowed(self, mock_policy, _):
        mock_policy.check.return_value = True
        self.assertTrue(self.deployment_tab.allowed(self.mock_request))
        mock_policy.check.assert_called_once_with(
            (("murano", "list_deployments"),), self.mock_request)

    def test_get_deployments_data(self, mock_api):
        mock_api.deployments_list.return_value = ['foo_deployment']
        deployments = self.deployment_tab.get_deployments_data()
        self.assertEqual(['foo_deployment'], deployments)
        mock_api.deployments_list.assert_called_once_with(
            self.mock_request, 'foo_env_id')

    @mock.patch.object(tabs, 'reverse')
    @mock.patch.object(tabs, 'exceptions')
    def test_get_deployments_data_except_http_forbidden(
            self, mock_exc, mock_reverse, mock_api):
        mock_api.deployments_list.side_effect = exc.HTTPForbidden
        mock_reverse.return_value = 'foo_reverse_url'

        self.deployment_tab.get_deployments_data()

        mock_exc.handle.assert_called_once_with(
            self.mock_request, _('Unable to retrieve list of deployments'),
            redirect='foo_reverse_url')
        mock_reverse.assert_called_once_with(
            "horizon:app-catalog:environments:index")

    @mock.patch.object(tabs, 'reverse')
    @mock.patch.object(tabs, 'exceptions')
    def test_get_deployments_data_except_http_server_error(
            self, mock_exc, mock_reverse, mock_api):
        mock_api.deployments_list.side_effect = exc.HTTPInternalServerError
        mock_reverse.return_value = 'foo_reverse_url'

        self.deployment_tab.get_deployments_data()

        mock_exc.handle.assert_called_once_with(
            self.mock_request,
            "Environment with id foo_env_id doesn't exist anymore",
            redirect='foo_reverse_url')
        mock_reverse.assert_called_once_with(
            "horizon:app-catalog:environments:index")


class TestEnvironmentDetailsTabs(unittest.TestCase):

    @mock.patch.object(tabs, 'api')
    def test_init(self, mock_api):
        mock_api.load_environment_data.return_value =\
            '{"environment": {"status": "foo_status"}}'

        mock_request = mock.Mock(GET={})
        mock_request.session = {'django_timezone': 'UTC'}
        mock_logs = mock.Mock(created='1970-01-01T12:34:00')
        mock_logs.created = '1970-01-01 12:34:00'
        env_details_tabs = tabs.EnvironmentDetailsTabs(
            mock_request, environment_id='foo_env_id', logs=[mock_logs])

        self.assertEqual('environment_details', env_details_tabs.slug)
        self.assertEqual(
            (tabs.EnvironmentServicesTab, tabs.EnvironmentTopologyTab,
             tabs.DeploymentTab, tabs.LatestLogsTab),
            env_details_tabs.tabs)
        self.assertTrue(env_details_tabs.sticky)


class TestServicesTabs(unittest.TestCase):

    def test_init(self):
        mock_request = mock.Mock(GET={})
        services_tabs = tabs.ServicesTabs(mock_request)

        self.assertEqual('services_details', services_tabs.slug)
        self.assertEqual(
            (tabs.OverviewTab, tabs.ServiceLogsTab),
            services_tabs.tabs)
        self.assertTrue(services_tabs.sticky)


class TestDeploymentDetailsTabs(unittest.TestCase):

    def test_init(self):
        mock_request = mock.Mock(GET={})
        deployment_details_tabs = tabs.DeploymentDetailsTabs(mock_request)

        self.assertEqual('deployment_details', deployment_details_tabs.slug)
        self.assertEqual((tabs.EnvConfigTab, tabs.EnvLogsTab,),
                         deployment_details_tabs.tabs)
        self.assertTrue(deployment_details_tabs.sticky)



#    Copyright (c) 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from django.template import loader
from django.templatetags.static import static
from django.urls import reverse

from muranodashboard.api import packages as pkg_cli
from muranodashboard.environments import consts


def get_app_image(request, app_fqdn, status=None):
    if '@' in app_fqdn:
        class_fqn, package_fqn = app_fqdn.split('@')
        if '/' in class_fqn:
            class_fqn, version = class_fqn.split('/')
        else:
            version = None
    else:
        package_fqn = app_fqdn
        version = None
    package = pkg_cli.app_by_fqn(request, package_fqn, version=version)
    if status in [
       consts.STATUS_ID_DEPLOY_FAILURE,
       consts.STATUS_ID_DELETE_FAILURE,
       ]:
        url = static('dashboard/img/stack-red.svg')
    elif status == consts.STATUS_ID_READY:
        url = static('dashboard/img/stack-green.svg')
    else:
        url = static('dashboard/img/stack-gray.svg')
    if package:
        app_id = package.id
        url = reverse("horizon:app-catalog:catalog:images", args=(app_id,))
    return url


def _get_environment_status_message(entity):
    if hasattr(entity, 'status'):
        status = entity.status
    else:
        status = entity['?']['status']

    in_progress = True
    status_message = ''
    if status in (consts.STATUS_ID_PENDING, consts.STATUS_ID_READY):
        in_progress = False
    if status == consts.STATUS_ID_PENDING:
        status_message = 'Waiting for deployment'
    elif status == consts.STATUS_ID_READY:
        status_message = 'Deployed'
    elif status == consts.STATUS_ID_DEPLOYING:
        status_message = 'Deployment is in progress'
    elif status == consts.STATUS_ID_DEPLOY_FAILURE:
        status_message = 'Deployment failed'
    return in_progress, status_message


def _truncate_type(type_str, num_of_chars):
    if len(type_str) < num_of_chars:
        return type_str
    else:
        parts = type_str.split('.')
        type_str, type_len = parts[-1], len(parts[-1])
        for part in reversed(parts[:-1]):
            if type_len + len(part) + 1 > num_of_chars:
                return '...' + type_str
            else:
                type_str = part + '.' + type_str
                type_len += len(part) + 1
        return type_str


def _application_info(application, app_image, status):
    name = application['?'].get('name')
    if not name:
        name = application.get('name')
    context = {'name': name,
               'type': _truncate_type(application['?']['type'], 45),
               'status': status,
               'app_image': app_image}
    return loader.render_to_string('services/_application_info.html',
                                   context)


def _network_info(name, image):
    context = {'name': name,
               'image': image}
    return loader.render_to_string('services/_network_info.html',
                                   context)


def _unit_info(unit, unit_image):
    data = dict(unit)
    data['type'] = _truncate_type(data['type'], 45)
    context = {'data': data,
               'unit_image': unit_image}

    return loader.render_to_string('services/_unit_info.html', context)


def _environment_info(environment, status):
    context = {'name': environment.name,
               'status': status}
    return loader.render_to_string('services/_environment_info.html',
                                   context)


def _create_empty_node():
    node = {
        'name': '',
        'status': 'ready',
        'image': '',
        'image_size': 60,
        'required_by': [],
        'image_x': -30,
        'image_y': -30,
        'text_x': 40,
        'text_y': ".35em",
        'link_type': "relation",
        'in_progress': False,
        'info_box': ''
    }
    return node


def _create_ext_network_node(name):
    node = _create_empty_node()
    node.update({'id': name,
                 'image': static('muranodashboard/images/ext-net.png'),
                 'link_type': 'relation',
                 'info_box': _network_info(name, static(
                     'dashboard/img/lb-green.svg'))}
                )
    return node


def _convert_lists(node_data):
    for key, value in node_data.items():
        if isinstance(value, list) and all(
                map(lambda s: not isinstance(s, (dict, list)), value)):
            new_value = ', '.join(str(v) for v in value)
            node_data[key] = new_value


def _split_seq_by_predicate(seq, predicate):
    holds, not_holds = [], []
    for elt in seq:
        if predicate(elt):
            holds.append(elt)
        else:
            not_holds.append(elt)
    return holds, not_holds


def _is_atomic(elt):
    key, value = elt
    return not isinstance(value, (dict, list))


def render_d3_data(request, environment):
    if not (environment and environment.services):
        return None

    ext_net_name = None
    d3_data = {"nodes": [], "environment": {}}

    in_progress, status_message = _get_environment_status_message(environment)
    environment_node = _create_empty_node()
    environment_node.update({
        'id': environment.id,
        'name': environment.name,
        'status': status_message,
        'image': static('dashboard/img/stack-green.svg'),
        'in_progress': in_progress,
        'info_box': _environment_info(environment, status_message)
    })
    d3_data['environment'] = environment_node

    unit_image_active = static('dashboard/img/server-green.svg')
    unit_image_non_active = static('dashboard/img/server-gray.svg')

    node_refs = {}

    def get_image(fqdn, node_data):
        if fqdn.startswith('io.murano.resources'):
            if len(node_data.get('ipAddresses', [])) > 0:
                image = unit_image_active
            else:
                image = unit_image_non_active
        else:
            image = get_app_image(request, fqdn)
        return image

    def rec(node_data, node_key, parent_node=None):
        if not isinstance(node_data, dict):
            return
        _convert_lists(node_data)
        node_type = node_data.get('?', {}).get('type')
        node_id = node_data.get('?', {}).get('id')
        atomics, containers = _split_seq_by_predicate(
            node_data.items(), _is_atomic)
        if node_type and node_data is not parent_node:
            node = _create_empty_node()
            node_refs[node_id] = node
            atomics.extend([('id', node_data['?']['id']),
                            ('type', node_type),
                            ('name', node_data.get('name', node_key))])

            image = get_image(node_type, node_data)
            node.update({
                'id': node_id,
                'info_box': _unit_info(atomics, image),
                'image': image,
                'link_type': 'unit',
                'in_progress': in_progress})
            d3_data['nodes'].append(node)

        for key, value in containers:
            if key == '?':
                continue
            if isinstance(value, dict):
                rec(value, key, node_data)
            elif isinstance(value, list):
                for index, val in enumerate(value):
                    rec(val, '{0}[{1}]'.format(key, index), node_data)

    def build_links_rec(node_data, parent_node=None):
        if not isinstance(node_data, dict):
            return
        node_id = node_data.get('?', {}).get('id')
        if not node_id:
            return
        node = node_refs[node_id]

        atomics, containers = _split_seq_by_predicate(
            node_data.items(), _is_atomic)

        # the actual second pass of node linking
        if parent_node is not None:
            node['required_by'].append(parent_node['?']['id'])
            node['link_type'] = 'aggregation'

        for key, value in atomics:
            if value in node_refs:
                remote_node = node_refs[value]
                if node_id not in remote_node['required_by']:
                    remote_node['required_by'].append(node_id)
                    remote_node['link_type'] = 'reference'

        for key, value in containers:
            if key == '?':
                continue
            if isinstance(value, dict):
                build_links_rec(value, node_data)
            elif isinstance(value, list):
                for val in value:
                    build_links_rec(val, node_data)

    for service in environment.services:
        in_progress, status_message = _get_environment_status_message(service)
        required_by = None
        if 'instance' in service and service['instance'] is not None:
            if service['instance'].get('assignFloatingIp', False):
                if ext_net_name:
                    required_by = ext_net_name
                else:
                    ext_net_name = 'External_Network'
                    ext_network_node = _create_ext_network_node(ext_net_name)
                    d3_data['nodes'].append(ext_network_node)
                    required_by = ext_net_name

        service_node = _create_empty_node()
        service_image = get_app_image(request, service['?']['type'],
                                      service['?']['status'])
        node_id = service['?']['id']
        node_refs[node_id] = service_node
        service_node.update({
            'name': service.get('name', ''),
            'status': status_message,
            'image': service_image,
            'id': node_id,
            'link_type': 'relation',
            'in_progress': in_progress,
            'info_box': _application_info(
                service, service_image, status_message)
        })
        if required_by:
            service_node['required_by'].append(required_by)
        d3_data['nodes'].append(service_node)
        rec(service, None, service)

    for service in environment.services:
        build_links_rec(service)

    return json.dumps(d3_data)



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import unittest
from unittest import mock

from castellan.common import exception as castellan_exception
from castellan.common.objects import opaque_data

from muranodashboard.dynamic_ui import helpers
from muranodashboard.dynamic_ui import yaql_functions


class TestYAQLFunctions(unittest.TestCase):

    def test_generate_hostname(self):
        self.assertEqual(
            yaql_functions._generate_hostname('foo-#', 1), 'foo-1')
        self.assertEqual(
            yaql_functions._generate_hostname('foo-#', 22), 'foo-22')

    def test_generate_hostname_random(self):
        random = yaql_functions._generate_hostname('', 3)
        self.assertTrue(bool(re.match(r'^\w{14}$', random)))

    def test_repeat(self):
        context = {}
        result = yaql_functions._repeat(context, 'foo_template', 2)
        self.assertEqual(['foo_template', 'foo_template'], [x for x in result])

    def test_name(self):
        context = mock.MagicMock()
        context.get_data.__getitem__.return_value = \
            {'application_name': 'foo_app'}
        self.assertEqual('foo_app', yaql_functions._name(context))

    def test_ref(self):
        parameters = {
            '#foo_template': {
                '?': {
                    'id': None
                }
            }
        }
        mock_service = mock.Mock(parameters=parameters)
        context = {'?service': mock_service}
        result = yaql_functions._ref(context, 'foo_template')
        self.assertIsInstance(result['?']['id'], helpers.ObjectID)

    def test_ref_with_id_only(self):
        object_id = helpers.ObjectID()
        parameters = {
            '#foo_template': {
                '?': {
                    'id': object_id
                }
            }
        }
        mock_service = mock.Mock(parameters=parameters)
        context = {'?service': mock_service}
        result = yaql_functions._ref(context, 'foo_template')
        self.assertIsInstance(result, helpers.ObjectID)
        self.assertEqual(object_id, result)

    def test_ref_with_evaluate_template(self):
        templates = {
            'foo_template': {
                '?': {
                    'id': helpers.ObjectID()
                }
            }
        }
        mock_service = mock.Mock(parameters={}, templates=templates)
        context = {'?service': mock_service}
        result = yaql_functions._ref(context, 'foo_template')
        self.assertIsInstance(result, helpers.ObjectID)

    def test_ref_return_none(self):
        mock_service = mock.Mock(parameters={'#foo_template': 'foo_data'})
        context = {'?service': mock_service}
        result = yaql_functions._ref(context, 'foo_template')
        self.assertIsNone(result)

    @mock.patch('muranodashboard.dynamic_ui.yaql_functions.settings')
    @mock.patch('muranodashboard.dynamic_ui.yaql_functions._oslo_context')
    @mock.patch('muranodashboard.dynamic_ui.yaql_functions.key_manager')
    @mock.patch('muranodashboard.dynamic_ui.yaql_functions.identity')
    def test_encrypt_data(self, mock_identity, mock_keymanager,
                          mock_oslo_context, _):
        mock_service = mock.Mock(parameters={'#foo_template': 'foo_data'})
        context = {'?service': mock_service}
        secret_value = 'secret_password'
        mock_auth_context = mock.MagicMock()
        mock_oslo_context.RequestContext.return_value = mock_auth_context
        yaql_functions._encrypt_data(context, secret_value)
        mock_keymanager.API().store.assert_called_once_with(
            mock_auth_context, opaque_data.OpaqueData(secret_value))

    def test_encrypt_data_not_configured(self):
        mock_service = mock.Mock(parameters={'#foo_template': 'foo_data'})
        context = {'?service': mock_service}
        self.assertRaises(castellan_exception.KeyManagerError,
                          yaql_functions._encrypt_data, context,
                          'secret_password')

    @mock.patch('muranodashboard.dynamic_ui.yaql_functions.identity')
    @mock.patch('muranodashboard.dynamic_ui.yaql_functions.settings')
    def test_encrypt_data_badly_configured(self, mock_settings, _):
        mock_service = mock.Mock(parameters={'#foo_template': 'foo_data'})
        context = {'?service': mock_service}
        mock_settings.KEY_MANAGER = {}
        self.assertRaises(castellan_exception.KeyManagerError,
                          yaql_functions._encrypt_data, context,
                          'secret_password')



#    Copyright (c) 2015 Mirantis, Inc.
#    Copyright (c) 2016 AT&T Corp
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
from django import forms
import semantic_version
from unittest import mock
from yaql.language import factory

from muranodashboard.catalog import forms as catalog_forms
from muranodashboard.catalog import views as catalog_views
from muranodashboard.dynamic_ui import forms as service_forms
from muranodashboard.dynamic_ui import services

from openstack_dashboard.test import helpers


class TestService(helpers.APITestCase):
    def setUp(self):
        super(TestService, self).setUp()

        self.application = {'?': {'type': 'test.App'}}
        factory = helpers.RequestFactoryWithMessages()
        self.request = factory.get('/path/for/testing')
        self.request.session = {}

    def test_service_field_hidden_false(self):
        """Test that service field is hidden

        When Service class instantiated with some field having `hidden`
        attribute set to `false` - the generated form field should have
        widget different from `HiddenInput`.

        Bug: #1368120
        """
        ui = [{
            'appConfiguration': {'fields': [{'hidden': False, 'type': 'string',
                                             'name': 'title'}]}
        }]

        service = services.Service(cleaned_data={},
                                   version=2,
                                   fqn='io.murano.Test',
                                   application=self.application,
                                   forms=ui)
        form = next(e for e in service.forms
                    if e.__name__ == 'appConfiguration')
        field = form.base_fields['title']

        self.assertNotIsInstance(field.widget, forms.HiddenInput)

    def test_service_field_hidden_true(self):
        """Test hidden widget

        `hidden: true` in UI definition results to HiddenInput in Django form.
        """
        ui = [{
            'appConfiguration': {'fields': [{'hidden': True, 'type': 'string',
                                             'name': 'title'}]}
        }]

        service = services.Service(cleaned_data={},
                                   version=2,
                                   fqn='io.murano.Test',
                                   application=self.application,
                                   forms=ui)
        form = next(e for e in service.forms
                    if e.__name__ == 'appConfiguration')
        field = form.base_fields['title']

        self.assertIsInstance(field.widget, forms.HiddenInput)

    def test_init_service_with_kwargs_and_version_coercion(self):
        kwargs = {'foo': 'bar', 'baz': 'qux', 'quux': 'corge'}
        service = services.Service(cleaned_data={},
                                   version=semantic_version.Version('2.3.0'),
                                   fqn='io.murano.Test',
                                   application=self.application,
                                   **kwargs)

        for key, val in kwargs.items():
            self.assertTrue(hasattr(service, key))
            self.assertEqual(val, getattr(service, key))

        self.assertEqual(1, len(service.forms))
        for key, val in kwargs.items():
            self.assertTrue(hasattr(service.forms[0].service, key))
            self.assertEqual(val, getattr(service.forms[0].service, key))

    def test_init_service_except_value_error(self):
        with self.assertRaisesRegex(ValueError,
                                    'Application section is required'):
            services.Service(cleaned_data={},
                             version=2,
                             fqn='io.murano.Test',
                             application=None)

    def test_extract_attributes(self):
        cleaned_data = {
            services.catalog_forms.WF_MANAGEMENT_NAME: {
                'application_name': 'foobar'
            }
        }
        templates = {'t1': 'foo', 't2': 'bar', 't3': 'baz'}
        service = services.Service(cleaned_data=cleaned_data,
                                   version=semantic_version.Version('2.3.0'),
                                   fqn='io.murano.Test',
                                   application=self.application,
                                   templates=templates)

        attributes = service.extract_attributes()
        expected = {'?': {'type': 'test.App', 'name': 'foobar'}}

        self.assertIsInstance(attributes, dict)
        self.assertEqual(expected['?']['type'], attributes['?']['type'])
        self.assertEqual(expected['?']['name'], attributes['?']['name'])
        self.assertEqual('foobar', service.application['?']['name'])

    def test_get_and_set_cleaned_data(self):
        cleaned_data = {
            catalog_forms.WF_MANAGEMENT_NAME: {
                'application_name': 'foobar'
            }
        }
        engine_factory = factory.YaqlFactory()
        engine = engine_factory.create()
        expr = engine('$')

        service = services.Service(cleaned_data={},
                                   version=semantic_version.Version('2.3.0'),
                                   fqn='io.murano.Test',
                                   application=self.application)
        service.set_data(cleaned_data)
        result = service.get_data(catalog_forms.WF_MANAGEMENT_NAME, expr)
        expected = {'workflowManagement': {'application_name': 'foobar'}}

        self.assertEqual(expected, result)

        # Test whether passing data to get_data works.
        service.set_data({})
        cleaned_data = cleaned_data[catalog_forms.WF_MANAGEMENT_NAME]
        result = service.get_data(
            catalog_forms.WF_MANAGEMENT_NAME, expr, data=cleaned_data)
        self.assertEqual(expected, result)

    def test_get_apps_data(self):
        result = services.get_apps_data(self.request)
        self.assertEqual({}, result)
        self.assertEqual(self.request.session['apps_data'], result)

    @mock.patch.object(services, 'pkg_api')
    def test_import_app(self, mock_pkg_api):
        mock_pkg_api.get_app_ui.return_value = {
            'foo': 'bar',
            'application': self.application
        }
        mock_pkg_api.get_app_fqn.return_value = 'foo_fqn'

        service = services.import_app(self.request, '123')
        self.assertEqual(services.Service, type(service))
        self.assertEqual('bar', service.foo)
        self.assertEqual(self.application, service.application)

    @mock.patch.object(services, 'pkg_api')
    def test_condition_getter_with_stay_at_the_catalog(self, mock_pkg_api):
        mock_pkg_api.get_app_ui.return_value = {
            'foo': 'bar',
            'application': self.application
        }

        service = services.Service(cleaned_data={},
                                   version=semantic_version.Version('2.2.1'),
                                   fqn='io.murano.Test',
                                   application=self.application)
        form = service_forms.ServiceConfigurationForm()
        form.service = service
        form.base_fields = {'stay_at_the_catalog': True}
        wizard = catalog_views.Wizard()
        wizard.kwargs = {'drop_wm_form': True}

        wizard.form_list = collections.OrderedDict({
            '123': form
        })

        kwargs = {'app_id': '123'}
        result = services.condition_getter(self.request, kwargs)
        self.assertIn('Step 1', result)
        self.assertIsNotNone(result['Step 1'])

        result = result['Step 1'](wizard)
        self.assertTrue(result)
        self.assertNotIn('stay_at_the_catalog', form.base_fields)

    @mock.patch.object(services, 'pkg_api')
    def test_condition_getter_with_application_name(self, mock_pkg_api):
        mock_pkg_api.get_app_ui.return_value = {
            'foo': 'bar',
            'application': self.application
        }

        service = services.Service(cleaned_data={},
                                   version=semantic_version.Version('2.1.9'),
                                   fqn='io.murano.Test',
                                   application=self.application)
        form = service_forms.ServiceConfigurationForm()
        form.service = service
        form.base_fields = {'application_name': 'foo_app_name'}
        wizard = catalog_views.Wizard()
        wizard.kwargs = {'drop_wm_form': False}

        wizard.form_list = collections.OrderedDict({
            '123': form
        })

        kwargs = {'app_id': '123'}
        result = services.condition_getter(self.request, kwargs)
        self.assertIn('Step 1', result)
        self.assertIsNotNone(result['Step 1'])

        result = result['Step 1'](wizard)
        self.assertTrue(result)
        self.assertNotIn('application_name', form.base_fields)

    @mock.patch.object(services, 'pkg_api')
    def test_condition_getter_with_form_hidden(self, mock_pkg_api):
        mock_pkg_api.get_app_ui.return_value = {
            'foo': 'bar',
            'application': self.application
        }

        service = services.Service(cleaned_data={},
                                   version=semantic_version.Version('2.1.9'),
                                   fqn='io.murano.Test',
                                   application=self.application)
        form = service_forms.ServiceConfigurationForm()
        form.service = service
        wizard = catalog_views.Wizard()
        wizard.kwargs = {'drop_wm_form': True}

        wizard.form_list = collections.OrderedDict({
            '123': form
        })

        kwargs = {'app_id': '123'}
        result = services.condition_getter(self.request, kwargs)
        self.assertIn('Step 1', result)
        self.assertIsNotNone(result['Step 1'])

        result = result['Step 1'](wizard)
        self.assertFalse(result)

    @mock.patch.object(services, 'pkg_api')
    def test_get_app_forms(self, mock_pkg_api):
        mock_pkg_api.get_app_ui.return_value = {'Application': {}}

        kwargs = {'app_id': '123'}
        result = services.get_app_forms(self.request, kwargs)
        self.assertIsNotNone(result)
        result = next(iter(result))
        self.assertEqual('Step 1', result[0])
        self.assertEqual(service_forms.DynamicFormMetaclass,
                         type(result[1]))

    def test_service_type_from_id(self):
        match_id = 'aaa123-345'
        non_match_ids = ['aaa', '-aaa', 'aaa123-aaa']

        result = services.service_type_from_id(match_id)
        self.assertEqual('aaa123', result)

        for non_match_id in non_match_ids:
            result = services.service_type_from_id(non_match_id)
            self.assertEqual(non_match_id, result)

    @mock.patch.object(services, 'import_app')
    def test_get_app_field_description(self, mock_import_app):
        form = service_forms.ServiceConfigurationForm()
        mock_field = mock.Mock(description_title='test_title',
                               description='test_description')
        mock_field.widget.is_hidden = False
        form.base_fields = {'foo': mock_field}
        mock_import_app.return_value = mock.Mock(forms=[form])

        result = services.get_app_field_descriptions(self.request, '123', 0)
        self.assertEqual(2, len(result))
        descriptions, no_field_descriptions = result

        self.assertEqual([('foo', 'test_title', 'test_description')],
                         descriptions)
        self.assertEqual([], no_field_descriptions)

    @mock.patch.object(services, 'import_app')
    def test_get_app_field_description_with_hidden_field(self,
                                                         mock_import_app):
        form = service_forms.ServiceConfigurationForm()
        mock_field = mock.Mock(description_title='test_title',
                               description='test_description')
        mock_field.widget.is_hidden = True
        form.base_fields = {'foo': mock_field}
        mock_import_app.return_value = mock.Mock(forms=[form])

        result = services.get_app_field_descriptions(self.request, '123', 0)
        self.assertEqual(2, len(result))
        descriptions, no_field_descriptions = result

        self.assertEqual(['test_description', 'test_title'],
                         sorted(no_field_descriptions))
        self.assertEqual([], descriptions)
